@@ -7,9 +7,11 @@
 
 set -eu
 
-FILES=$(find src -type f -type f \( -iname "*.cpp" -o -iname "*.h" \))
+FILES=$(find src -type f \( -iname "*.cpp" -o -iname "*.h" \))
 
 for f in $FILES
 do
-    clang-format -i "$f" && git diff --exit-code
+    clang-format -i "$f"
 done;
+
+git diff --exit-code
