@@ -7,24 +7,28 @@ set -u
 
 # Add Qt binaries to path
 PATH="/usr/local/opt/qt@5/bin/:${PATH}"
+export PATH
 
 security unlock-keychain -p "${RUNNER_USER_PW}" login.keychain
 
-( cd build || exit
-  # macdeployqt does not copy symlinks over.
-  # this specifically addresses icu4c issues but nothing else.
-  # We might not even need this any longer... 
-  # ICU_LIB="$(brew --prefix icu4c)/lib"
-  # export ICU_LIB
-  # mkdir -p nheko.app/Contents/Frameworks
-  # find "${ICU_LIB}" -type l -name "*.dylib" -exec cp -a -n {} nheko.app/Contents/Frameworks/ \; || true
+if [ -n "${CI_PIPELINE_TRIGGERED:-}" ] && [ "${TRIGGERED_BY:-}" = "cirrus" ]; then
+  echo "cirrus build id: ${TRIGGER_BUILD_ID}"
+  cat "${TRIGGER_PAYLOAD}"
+  # download the build artifacts from cirrus api
+  curl "https://api.cirrus-ci.com/v1/artifact/build/${TRIGGER_BUILD_ID}/binaries.zip" -o binaries.zip
+  # cirrus ci artifacts task name is 'binaries' so that's the zip name.
+  unzip binaries.zip
+  # we zip 'build/nheko.app' in cirrus ci, cirrus itself puts it in a 'build' directory
+  # so move it to the right place for the rest of the process.
+  ( cd build || exit
+    unzip nheko.zip
+  )
+fi
 
-  #macdeployqt nheko.app -dmg -always-overwrite -qmldir=../resources/qml/ -sign-for-notarization="${APPLE_DEV_IDENTITY}"
-  macdeployqt nheko.app -always-overwrite -qmldir=../resources/qml/
-
-  # user=$(id -nu)
-  # chown "${user}" nheko.dmg
-)
+if [ ! -d "build/nheko.app" ]; then
+  echo "nheko.app is missing, you did something wrong!"
+  exit 1
+fi
 
 echo "[INFO] Signing app contents"
 find "build/nheko.app/Contents"|while read -r fname; do
@@ -46,6 +50,7 @@ trap finish EXIT
 
 dmgbuild -s .ci/macos/settings.json "Nheko" nheko.dmg
 codesign -s "${APPLE_DEV_IDENTITY}" nheko.dmg
+
 user=$(id -nu)
 chown "${user}" nheko.dmg
 
@@ -72,7 +77,7 @@ while sleep 60 && date; do
   #isSuccess=$(grep "success" "$NOTARIZE_STATUS_LOG")
   #isFailure=$(grep "invalid" "$NOTARIZE_STATUS_LOG")
 
-  echo "Status for submission \"${requestUUID}\": \"${sub_status}\"" 
+  echo "Status for submission \"${requestUUID}\": \"${sub_status}\""
 
   if [ "${sub_status}" = "Accepted" ]; then
       echo "Notarization done!"
@@ -82,6 +87,7 @@ while sleep 60 && date; do
   fi
   if [ "${sub_status}" = "Invalid" ] || [ "${sub_status}" = "Rejected" ]; then
       echo "Notarization failed"
+      xcrun notarytool log "${requestUUID}" --apple-id "${APPLE_DEV_USER}" --password "${APPLE_DEV_PASS}" --team-id "${APPLE_TEAM_ID}" > "$NOTARIZE_STATUS_LOG" 2>&1
       cat "$NOTARIZE_STATUS_LOG" 1>&2
       exit 1
   fi
@@ -91,7 +97,7 @@ done
 VERSION=${CI_COMMIT_SHORT_SHA}
 
 if [ -n "$VERSION" ]; then
-    mv nheko.dmg "nheko-${VERSION}.dmg"
+    mv nheko.dmg "nheko-${VERSION}_${PLAT}.dmg"
     mkdir artifacts
-    cp "nheko-${VERSION}.dmg" artifacts/
+    cp "nheko-${VERSION}_${PLAT}.dmg" artifacts/
 fi
\ No newline at end of file
