@@ -13,17 +13,18 @@ class Emoji(object):
         self.unicodename = unicodename
 
 def generate_qml_list(**kwargs):
+    entrycount = sum([len(c[1]) for c in kwargs.items()])
     tmpl = Template('''
-const std::array<Emoji, {{ sum([len(c[1]) for c in kwargs.items()]) }}> emoji::Provider::emoji = {
+constexpr const std::array<Emoji, {{ entrycount }} > emoji::Provider::emoji = {
     {%- for c in kwargs.items() %}
     // {{ c[0].capitalize() }}
     {%- for e in c[1] %}
-    Emoji{QStringLiteral(u"{{ e.code }}"), QStringLiteral(u"{{ e.shortname }}"), QStringLiteral(u"{{ e.unicodename }}"), emoji::Emoji::Category::{{ c[0].capitalize() }}},
+    Emoji{std::u16string_view(u"{{ e.code }}"), std::u16string_view(u"{{ e.shortname }}"), std::u16string_view(u"{{ e.unicodename }}"), emoji::Emoji::Category::{{ c[0].capitalize() }}},
     {%- endfor %}
     {%- endfor %}
 };
     ''')
-    d = dict(kwargs=kwargs)
+    d = dict(kwargs=kwargs, entrycount=entrycount)
     print(tmpl.render(d))
 if __name__ == '__main__':
     if len(sys.argv) < 3:
@@ -54,9 +55,9 @@ if __name__ == '__main__':
         'Flags': flags,
         'Component': symbols
     }
-    shortcodeDict = {} 
+    shortcodeDict = {}
     # for my sanity - this strips newlines
-    for line in open(shortcodefilename, 'r', encoding="utf8"): 
+    for line in open(shortcodefilename, 'r', encoding="utf8"):
         longname, shortname = line.strip().split(':')
         shortcodeDict[longname] = shortname
     current_category = ''
