@@ -2,6 +2,7 @@
 // SPDX-FileCopyrightText: 2019 The nheko authors
 // SPDX-FileCopyrightText: 2021 Nheko Contributors
 // SPDX-FileCopyrightText: 2022 Nheko Contributors
+// SPDX-FileCopyrightText: 2023 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -33,7 +34,7 @@ namespace mtx::responses {
 struct Messages;
 }
 
-class Cache : public QObject
+class Cache final : public QObject
 {
     Q_OBJECT
 
@@ -118,7 +119,9 @@ public:
                                                  std::size_t len        = 30);
     size_t memberCount(const std::string &room_id);
 
-    void updateState(const std::string &room, const mtx::responses::StateEvents &state);
+    void updateState(const std::string &room,
+                     const mtx::responses::StateEvents &state,
+                     bool wipe = false);
     void saveState(const mtx::responses::Sync &res);
     bool isInitialized();
     bool isDatabaseReady() { return databaseReady_ && isInitialized(); }
@@ -138,7 +141,6 @@ public:
     bool runMigrations();
 
     std::vector<QString> roomIds();
-    QMap<QString, mtx::responses::Notifications> getTimelineMentions();
 
     //! Retrieve all the user ids from a room.
     std::vector<std::string> roomMembers(const std::string &room_id);
@@ -165,10 +167,14 @@ public:
     RoomInfo singleRoomInfo(const std::string &room_id);
     std::vector<std::string> roomsWithStateUpdates(const mtx::responses::Sync &res);
     std::map<QString, RoomInfo> getRoomInfo(const std::vector<std::string> &rooms);
+
+    std::vector<RoomNameAlias> roomNamesAndAliases();
+
     void updateLastMessageTimestamp(const std::string &room_id, uint64_t ts);
 
     //! Calculates which the read status of a room.
     //! Whether all the events in the timeline have been read.
+    std::string getFullyReadEventId(const std::string &room_id);
     bool calculateRoomReadStatus(const std::string &room_id);
     void calculateRoomReadStatus();
 
@@ -178,9 +184,6 @@ public:
     //! Check if we have sent a desktop notification for the given event id.
     bool isNotificationSent(const std::string &event_id);
 
-    //! Add all notifications containing a user mention to the db.
-    void saveTimelineMentions(const mtx::responses::Notifications &res);
-
     //! retrieve events in timeline and related functions
     struct Messages
     {
@@ -194,7 +197,7 @@ public:
                                  bool forward   = false);
 
     std::optional<mtx::events::collections::TimelineEvent>
-    getEvent(const std::string &room_id, const std::string &event_id);
+    getEvent(const std::string &room_id, std::string_view event_id);
     void storeEvent(const std::string &room_id,
                     const std::string &event_id,
                     const mtx::events::collections::TimelineEvent &event);
@@ -212,8 +215,9 @@ public:
     std::optional<uint64_t> getEventIndex(const std::string &room_id, std::string_view event_id);
     std::optional<std::pair<uint64_t, std::string>>
     lastInvisibleEventAfter(const std::string &room_id, std::string_view event_id);
+    std::optional<std::pair<uint64_t, std::string>>
+    lastVisibleEvent(const std::string &room_id, std::string_view event_id);
     std::optional<std::string> getTimelineEventId(const std::string &room_id, uint64_t index);
-    std::optional<uint64_t> getArrivalIndex(const std::string &room_id, std::string_view event_id);
 
     std::string previousBatchToken(const std::string &room_id);
     uint64_t saveOldMessages(const std::string &room_id, const mtx::responses::Messages &res);
@@ -246,6 +250,8 @@ public:
 
     //! Check if a user is a member of the room.
     bool isRoomMember(const std::string &user_id, const std::string &room_id);
+    std::optional<MemberInfo>
+    getInviteMember(const std::string &room_id, const std::string &user_id);
 
     //
     // Outbound Megolm Sessions
@@ -279,6 +285,8 @@ public:
     void saveOlmSession(const std::string &curve25519,
                         mtx::crypto::OlmSessionPtr session,
                         uint64_t timestamp);
+    void saveOlmSessions(std::vector<std::pair<std::string, mtx::crypto::OlmSessionPtr>> sessions,
+                         uint64_t timestamp);
     std::vector<std::string> getOlmSessions(const std::string &curve25519);
     std::optional<mtx::crypto::OlmSessionPtr>
     getOlmSession(const std::string &curve25519, const std::string &session_id);
@@ -291,9 +299,9 @@ public:
     void deleteBackupVersion();
     std::optional<OnlineBackupVersion> backupVersion();
 
-    void storeSecret(const std::string name, const std::string secret, bool internal = false);
-    void deleteSecret(const std::string name, bool internal = false);
-    std::optional<std::string> secret(const std::string name, bool internal = false);
+    void storeSecret(const std::string &name, const std::string &secret, bool internal = false);
+    void deleteSecret(const std::string &name, bool internal = false);
+    std::optional<std::string> secret(const std::string &name, bool internal = false);
 
     std::string pickleSecret();
 
@@ -316,7 +324,6 @@ public:
 signals:
     void newReadReceipts(const QString &room_id, const std::vector<QString> &event_ids);
     void roomReadStatus(const std::map<QString, bool> &status);
-    void removeNotification(const QString &room_id, const QString &event_id);
     void userKeysUpdate(const std::string &sync_token, const mtx::responses::QueryKeys &keyQuery);
     void userKeysUpdateFinalize(const std::string &user_id);
     void verificationStatusChanged(const std::string &userid);
@@ -325,7 +332,13 @@ signals:
     void databaseReady();
 
 private:
-    void loadSecrets(std::vector<std::pair<std::string, bool>> toLoad);
+    void loadSecretsFromStore(
+      std::vector<std::pair<std::string, bool>> toLoad,
+      std::function<void(const std::string &name, bool internal, const std::string &value)>
+        callback,
+      bool databaseReadyOnFinished = false);
+    void storeSecretInStore(const std::string name, const std::string secret);
+    void deleteSecretFromStore(const std::string name, bool internal);
 
     //! Save an invited room.
     void saveInvite(lmdb::txn &txn,
@@ -333,15 +346,6 @@ private:
                     lmdb::dbi &membersdb,
                     const mtx::responses::InvitedRoom &room);
 
-    //! Add a notification containing a user mention to the db.
-    void saveTimelineMentions(lmdb::txn &txn,
-                              const std::string &room_id,
-                              const QList<mtx::responses::Notification> &res);
-
-    //! Get timeline items that a user was mentions in for a given room
-    mtx::responses::Notifications
-    getTimelineMentionsForRoom(lmdb::txn &txn, const std::string &room_id);
-
     QString getInviteRoomName(lmdb::txn &txn, lmdb::dbi &statesdb, lmdb::dbi &membersdb);
     QString getInviteRoomTopic(lmdb::txn &txn, lmdb::dbi &statesdb);
     QString getInviteRoomAvatarUrl(lmdb::txn &txn, lmdb::dbi &statesdb, lmdb::dbi &membersdb);
@@ -401,7 +405,7 @@ private:
                   e->content.display_name.empty() ? e->state_key : e->content.display_name;
 
                 // Lightweight representation of a member.
-                MemberInfo tmp{display_name, e->content.avatar_url};
+                MemberInfo tmp{display_name, e->content.avatar_url, e->content.reason};
 
                 membersdb.put(txn, e->state_key, nlohmann::json(tmp).dump());
                 break;
@@ -412,6 +416,8 @@ private:
             }
             }
 
+            // BUG(Nico): Ideally we would fall through and store this in the database, but it seems
+            // to currently corrupt the db sometimes, so... let's find that bug first!
             return;
         } else if (std::holds_alternative<StateEvent<Encryption>>(event)) {
             setEncryptedRoom(txn, room_id);
@@ -488,13 +494,13 @@ private:
                     if (!eventsDb.get(
                           txn, nlohmann::json::parse(data)["id"].get<std::string>(), value))
                         return std::nullopt;
-                } catch (std::exception &e) {
+                } catch (std::exception &) {
                     return std::nullopt;
                 }
             }
 
             return nlohmann::json::parse(value).get<mtx::events::StateEvent<T>>();
-        } catch (std::exception &e) {
+        } catch (std::exception &) {
             return std::nullopt;
         }
     }
@@ -640,11 +646,6 @@ private:
         return lmdb::dbi::open(txn, std::string(room_id + "/members").c_str(), MDB_CREATE);
     }
 
-    lmdb::dbi getMentionsDb(lmdb::txn &txn, const std::string &room_id)
-    {
-        return lmdb::dbi::open(txn, std::string(room_id + "/mentions").c_str(), MDB_CREATE);
-    }
-
     lmdb::dbi getUserKeysDb(lmdb::txn &txn) { return lmdb::dbi::open(txn, "user_key", MDB_CREATE); }
 
     lmdb::dbi getVerificationDb(lmdb::txn &txn)
@@ -700,7 +701,6 @@ private:
     std::string pickle_secret_;
 
     VerificationStorage verification_storage;
-    SecretsStorage secret_storage;
 
     bool databaseReady_ = false;
 };
