@@ -1,6 +1,7 @@
 // SPDX-FileCopyrightText: 2017 Konstantinos Sideris <siderisk@auth.gr>
 // SPDX-FileCopyrightText: 2021 Nheko Contributors
 // SPDX-FileCopyrightText: 2022 Nheko Contributors
+// SPDX-FileCopyrightText: 2023 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -33,10 +34,10 @@
 
 #include "blurhash.hpp"
 
-ChatPage *ChatPage::instance_             = nullptr;
-constexpr int CHECK_CONNECTIVITY_INTERVAL = 15'000;
-constexpr int RETRY_TIMEOUT               = 5'000;
-constexpr size_t MAX_ONETIME_KEYS         = 50;
+ChatPage *ChatPage::instance_                    = nullptr;
+static constexpr int CHECK_CONNECTIVITY_INTERVAL = 15'000;
+static constexpr int RETRY_TIMEOUT               = 5'000;
+static constexpr size_t MAX_ONETIME_KEYS         = 50;
 
 Q_DECLARE_METATYPE(std::optional<mtx::crypto::EncryptedFile>)
 Q_DECLARE_METATYPE(std::optional<RelatedInfo>)
@@ -136,18 +137,6 @@ ChatPage::ChatPage(QSharedPointer<UserSettings> userSettings, QObject *parent)
             Qt::QueuedConnection);
     connect(this, &ChatPage::leftRoom, this, &ChatPage::removeRoom);
     connect(this, &ChatPage::changeToRoom, this, &ChatPage::changeRoom, Qt::QueuedConnection);
-    connect(this, &ChatPage::notificationsRetrieved, this, &ChatPage::sendNotifications);
-    connect(this,
-            &ChatPage::highlightedNotifsRetrieved,
-            this,
-            [](const mtx::responses::Notifications &notif) {
-                try {
-                    cache::saveTimelineMentions(notif);
-                } catch (const lmdb::error &e) {
-                    nhlog::db()->error("failed to save mentions: {}", e.what());
-                }
-            });
-
     connect(notificationsManager,
             &NotificationsManager::notificationClicked,
             this,
@@ -155,25 +144,20 @@ ChatPage::ChatPage(QSharedPointer<UserSettings> userSettings, QObject *parent)
                 Q_UNUSED(eventid)
                 auto exWin = MainWindow::instance()->windowForRoom(roomid);
                 if (exWin) {
+                    exWin->setVisible(true);
+                    exWin->raise();
                     exWin->requestActivate();
                 } else {
                     view_manager_->rooms()->setCurrentRoom(roomid);
+                    MainWindow::instance()->setVisible(true);
+                    MainWindow::instance()->raise();
                     MainWindow::instance()->requestActivate();
                 }
             });
     connect(notificationsManager,
             &NotificationsManager::sendNotificationReply,
             this,
-            [this](const QString &roomid, const QString &eventid, const QString &body) {
-                view_manager_->queueReply(roomid, eventid, body);
-                auto exWin = MainWindow::instance()->windowForRoom(roomid);
-                if (exWin) {
-                    exWin->requestActivate();
-                } else {
-                    view_manager_->rooms()->setCurrentRoom(roomid);
-                    MainWindow::instance()->requestActivate();
-                }
-            });
+            &ChatPage::sendNotificationReply);
 
     connect(
       this,
@@ -191,7 +175,8 @@ ChatPage::ChatPage(QSharedPointer<UserSettings> userSettings, QObject *parent)
         static unsigned int prevNotificationCount = 0;
         unsigned int notificationCount            = 0;
         for (const auto &room : sync.rooms.join) {
-            notificationCount += room.second.unread_notifications.notification_count;
+            notificationCount +=
+              static_cast<unsigned int>(room.second.unread_notifications.notification_count);
         }
 
         // HACK: If we had less notifications last time we checked, send an alert if the
@@ -208,18 +193,168 @@ ChatPage::ChatPage(QSharedPointer<UserSettings> userSettings, QObject *parent)
         // No need to check amounts for this section, as this function internally checks for
         // duplicates.
         if (notificationCount && userSettings_->hasNotifications())
-            http::client()->notifications(
-              5,
-              "",
-              "",
-              [this](const mtx::responses::Notifications &res, mtx::http::RequestErr err) {
-                  if (err) {
-                      nhlog::net()->warn("failed to retrieve notifications: {}", err);
-                      return;
-                  }
+            for (const auto &e : sync.account_data.events) {
+                if (auto newRules =
+                      std::get_if<mtx::events::AccountDataEvent<mtx::pushrules::GlobalRuleset>>(&e))
+                    pushrules =
+                      std::make_unique<mtx::pushrules::PushRuleEvaluator>(newRules->content.global);
+            }
+        if (!pushrules) {
+            auto eventInDb = cache::client()->getAccountData(mtx::events::EventType::PushRules);
+            if (eventInDb) {
+                if (auto newRules =
+                      std::get_if<mtx::events::AccountDataEvent<mtx::pushrules::GlobalRuleset>>(
+                        &*eventInDb))
+                    pushrules =
+                      std::make_unique<mtx::pushrules::PushRuleEvaluator>(newRules->content.global);
+            }
+        }
+        if (pushrules) {
+            const auto local_user = utils::localUser().toStdString();
 
-                  emit notificationsRetrieved(std::move(res));
-              });
+            for (const auto &[room_id, room] : sync.rooms.join) {
+                // clear old notifications
+                for (const auto &e : room.ephemeral.events) {
+                    if (auto receiptsEv =
+                          std::get_if<mtx::events::EphemeralEvent<mtx::events::ephemeral::Receipt>>(
+                            &e)) {
+                        std::vector<QString> receipts;
+
+                        for (const auto &[event_id, userReceipts] : receiptsEv->content.receipts) {
+                            if (auto r = userReceipts.find(mtx::events::ephemeral::Receipt::Read);
+                                r != userReceipts.end()) {
+                                for (const auto &[user_id, receipt] : r->second.users) {
+                                    (void)receipt;
+
+                                    if (user_id == local_user) {
+                                        receipts.push_back(QString::fromStdString(event_id));
+                                        break;
+                                    }
+                                }
+                            }
+                            if (auto r =
+                                  userReceipts.find(mtx::events::ephemeral::Receipt::ReadPrivate);
+                                r != userReceipts.end()) {
+                                for (const auto &[user_id, receipt] : r->second.users) {
+                                    (void)receipt;
+
+                                    if (user_id == local_user) {
+                                        receipts.push_back(QString::fromStdString(event_id));
+                                        break;
+                                    }
+                                }
+                            }
+                        }
+                        if (!receipts.empty())
+                            notificationsManager->removeNotifications(
+                              QString::fromStdString(room_id), receipts);
+                    }
+                }
+
+                // calculate new notifications
+                if (!room.timeline.events.empty() &&
+                    (room.unread_notifications.notification_count ||
+                     room.unread_notifications.highlight_count)) {
+                    auto roomModel =
+                      view_manager_->rooms()->getRoomById(QString::fromStdString(room_id));
+
+                    if (!roomModel) {
+                        continue;
+                    }
+
+                    auto currentReadMarker =
+                      cache::getEventIndex(room_id, cache::client()->getFullyReadEventId(room_id));
+
+                    auto ctx = roomModel->pushrulesRoomContext();
+                    std::vector<
+                      std::pair<mtx::common::Relation, mtx::events::collections::TimelineEvent>>
+                      relatedEvents;
+
+                    for (const auto &event : room.timeline.events) {
+                        auto event_id = mtx::accessors::event_id(event);
+
+                        // skip already read events
+                        if (currentReadMarker &&
+                            currentReadMarker > cache::getEventIndex(room_id, event_id))
+                            continue;
+
+                        // skip our messages
+                        auto sender = mtx::accessors::sender(event);
+                        if (sender == http::client()->user_id().to_string())
+                            continue;
+
+                        mtx::events::collections::TimelineEvent te{event};
+                        std::visit([room_id = room_id](auto &event_) { event_.room_id = room_id; },
+                                   te.data);
+
+                        if (auto encryptedEvent =
+                              std::get_if<mtx::events::EncryptedEvent<mtx::events::msg::Encrypted>>(
+                                &event);
+                            encryptedEvent && userSettings_->decryptNotifications()) {
+                            MegolmSessionIndex index(room_id, encryptedEvent->content);
+
+                            auto result = olm::decryptEvent(index, *encryptedEvent);
+                            if (result.event)
+                                te.data = result.event.value();
+                        }
+
+                        relatedEvents.clear();
+                        for (const auto &r : mtx::accessors::relations(te.data).relations) {
+                            auto related = cache::client()->getEvent(room_id, r.event_id);
+                            if (related) {
+                                relatedEvents.emplace_back(r, *related);
+                                if (auto encryptedEvent = std::get_if<
+                                      mtx::events::EncryptedEvent<mtx::events::msg::Encrypted>>(
+                                      &related->data);
+                                    encryptedEvent && userSettings_->decryptNotifications()) {
+                                    MegolmSessionIndex index(room_id, encryptedEvent->content);
+
+                                    auto result = olm::decryptEvent(index, *encryptedEvent);
+                                    if (result.event)
+                                        relatedEvents.back().second.data = result.event.value();
+                                }
+                            }
+                        }
+
+                        auto actions = pushrules->evaluate(te, ctx, relatedEvents);
+                        if (std::find(actions.begin(),
+                                      actions.end(),
+                                      mtx::pushrules::actions::Action{
+                                        mtx::pushrules::actions::notify{}}) != actions.end()) {
+                            if (!cache::isNotificationSent(event_id)) {
+                                // We should only send one notification per event.
+                                cache::markSentNotification(event_id);
+
+                                // Don't send a notification when the current room is opened.
+                                if (isRoomActive(roomModel->roomId()))
+                                    continue;
+
+                                if (userSettings_->hasDesktopNotifications()) {
+                                    auto info = cache::singleRoomInfo(room_id);
+
+                                    AvatarProvider::resolve(
+                                      roomModel->roomAvatarUrl(),
+                                      96,
+                                      this,
+                                      [this, te, room_id = room_id, actions](QPixmap image) {
+                                          notificationsManager->postNotification(
+                                            mtx::responses::Notification{
+                                              .actions     = actions,
+                                              .event       = te.data,
+                                              .read        = false,
+                                              .profile_tag = "",
+                                              .room_id     = room_id,
+                                              .ts          = 0,
+                                            },
+                                            image.toImage());
+                                      });
+                                }
+                            }
+                        }
+                    }
+                }
+            }
+        }
     });
 
     connect(
@@ -252,6 +387,9 @@ ChatPage::ChatPage(QSharedPointer<UserSettings> userSettings, QObject *parent)
     connectCallMessage<mtx::events::voip::CallCandidates>();
     connectCallMessage<mtx::events::voip::CallAnswer>();
     connectCallMessage<mtx::events::voip::CallHangUp>();
+    connectCallMessage<mtx::events::voip::CallSelectAnswer>();
+    connectCallMessage<mtx::events::voip::CallReject>();
+    connectCallMessage<mtx::events::voip::CallNegotiate>();
 }
 
 void
@@ -272,6 +410,21 @@ ChatPage::dropToLoginPage(const QString &msg)
     http::client()->shutdown();
     connectivityTimer_.stop();
 
+    auto btn = QMessageBox::warning(
+      nullptr,
+      tr("Confirm logout"),
+      tr("Because of the following reason Nheko wants to drop you to the login page:\n%1\nIf you "
+         "think this is a mistake, you can close Nheko instead to possibly recover your encryption "
+         "keys. After you have been dropped to the login page, you can sign in again using your "
+         "usual methods.")
+        .arg(msg),
+      QMessageBox::StandardButton::Close | QMessageBox::StandardButton::Ok,
+      QMessageBox::StandardButton::Ok);
+    if (btn == QMessageBox::StandardButton::Close) {
+        QCoreApplication::exit(1);
+        exit(1);
+    }
+
     resetUI();
     deleteConfigs();
 
@@ -350,9 +503,9 @@ ChatPage::bootstrap(QString userid, QString homeserver, QString token)
                               tr("Cache migration failed!"),
                               tr("Migrating the cache to the current version failed. "
                                  "This can have different reasons. Please open an "
-                                 "issue and try to use an older version in the mean "
-                                 "time. Alternatively you can try deleting the cache "
-                                 "manually."));
+                                 "issue at https://github.com/Nheko-Reborn/nheko and try to use an "
+                                 "older version in the meantime. Alternatively you can try "
+                                 "deleting the cache manually."));
                             QCoreApplication::quit();
                         }
                         loadStateFromCache();
@@ -395,11 +548,11 @@ ChatPage::bootstrap(QString userid, QString homeserver, QString token)
                 view_manager_,
                 &TimelineViewManager::updateReadReceipts);
 
-        connect(cache::client(),
-                &Cache::removeNotification,
-                notificationsManager,
-                &NotificationsManager::removeNotification);
-
+        connect(cache::client(), &Cache::secretChanged, this, [this](const std::string &secret) {
+            if (secret == mtx::secret_storage::secrets::megolm_backup_v1) {
+                getBackupVersion();
+            }
+        });
     } catch (const lmdb::error &e) {
         nhlog::db()->critical("failure during boot: {}", e.what());
         emit dropToLoginPageCb(tr("Failed to open database, logging out!"));
@@ -415,7 +568,6 @@ ChatPage::loadStateFromCache()
         olm::client()->load(cache::restoreOlmAccount(), cache::client()->pickleSecret());
 
         emit initializeEmptyViews();
-        emit initializeMentions(cache::getTimelineMentions());
 
         cache::calculateRoomReadStatus();
 
@@ -464,46 +616,6 @@ ChatPage::removeRoom(const QString &room_id)
     }
 }
 
-void
-ChatPage::sendNotifications(const mtx::responses::Notifications &res)
-{
-    for (const auto &item : res.notifications) {
-        const auto event_id = mtx::accessors::event_id(item.event);
-
-        try {
-            if (item.read) {
-                cache::removeReadNotification(event_id);
-                continue;
-            }
-
-            if (!cache::isNotificationSent(event_id)) {
-                const auto room_id = QString::fromStdString(item.room_id);
-
-                // We should only send one notification per event.
-                cache::markSentNotification(event_id);
-
-                // Don't send a notification when the current room is opened.
-                if (isRoomActive(room_id))
-                    continue;
-
-                if (userSettings_->hasDesktopNotifications()) {
-                    auto info = cache::singleRoomInfo(item.room_id);
-
-                    AvatarProvider::resolve(QString::fromStdString(info.avatar_url),
-                                            96,
-                                            this,
-                                            [this, item](QPixmap image) {
-                                                notificationsManager->postNotification(
-                                                  item, image.toImage());
-                                            });
-                }
-            }
-        } catch (const lmdb::error &e) {
-            nhlog::db()->warn("error while sending notification: {}", e.what());
-        }
-    }
-}
-
 void
 ChatPage::tryInitialSync()
 {
@@ -596,7 +708,6 @@ ChatPage::startInitialSync()
                 olm::handle_to_device_messages(res.to_device.events);
 
                 emit initializeViews(std::move(res));
-                emit initializeMentions(cache::getTimelineMentions());
 
                 cache::calculateRoomReadStatus();
             } catch (const lmdb::error &e) {
@@ -619,7 +730,7 @@ ChatPage::handleSyncResponse(const mtx::responses::Sync &res, const std::string
             nhlog::net()->warn("Duplicate sync, dropping");
             return;
         }
-    } catch (const lmdb::error &e) {
+    } catch (const lmdb::error &) {
         nhlog::db()->warn("Logged out in the mean time, dropping sync");
         return;
     }
@@ -709,7 +820,7 @@ ChatPage::knockRoom(const QString &room,
           tr("Knock on room"),
           // clang-format off
       failedJoin
-        ? tr("You failed to join %1. You can try to knock, so that others can invite you in. Do you want to do so?\nYou may optionally provide a reason for others to accept your knock:").arg(room)
+        ? tr("You failed to join %1. You can try to knock so that others can invite you in. Do you want to do so?\nYou may optionally provide a reason for others to accept your knock:").arg(room)
         : tr("Do you really want to knock on %1? You may optionally provide a reason for others to accept your knock:").arg(room),
           // clang-format on
           QLineEdit::Normal,
@@ -998,13 +1109,16 @@ ChatPage::verifyOneTimeKeyCountAfterStartup()
           }
 
           std::map<std::string, uint16_t> key_counts;
-          auto count = 0;
+          std::uint64_t count = 0;
           if (auto c = res.one_time_key_counts.find(mtx::crypto::SIGNED_CURVE25519);
               c == res.one_time_key_counts.end()) {
               key_counts[mtx::crypto::SIGNED_CURVE25519] = 0;
           } else {
-              key_counts[mtx::crypto::SIGNED_CURVE25519] = c->second;
-              count                                      = c->second;
+              key_counts[mtx::crypto::SIGNED_CURVE25519] =
+                c->second > std::numeric_limits<std::uint16_t>::max()
+                  ? std::numeric_limits<std::uint16_t>::max()
+                  : static_cast<std::uint16_t>(c->second);
+              count = c->second;
           }
 
           nhlog::crypto()->info(
@@ -1116,7 +1230,7 @@ ChatPage::getBackupVersion()
           }
 
           // switch to UI thread for secrets stuff
-          QTimer::singleShot(0, this, [res] {
+          QTimer::singleShot(0, this, [this, res] {
               auto auth_data = nlohmann::json::parse(res.auth_data);
 
               if (res.algorithm == "m.megolm_backup.v1.curve25519-aes-sha2") {
@@ -1139,11 +1253,17 @@ ChatPage::getBackupVersion()
                       return;
                   }
 
+                  auto oldBackupVersion = cache::client()->backupVersion();
+
                   nhlog::crypto()->info("Using online key backup.");
                   OnlineBackupVersion data{};
                   data.algorithm = res.algorithm;
                   data.version   = res.version;
                   cache::client()->saveBackupVersion(data);
+
+                  if (!oldBackupVersion || oldBackupVersion->version != data.version) {
+                      view_manager_->rooms()->refetchOnlineKeyBackupKeys();
+                  }
               } else {
                   nhlog::crypto()->info("Unsupported key backup algorithm: {}", res.algorithm);
                   cache::client()->deleteBackupVersion();
@@ -1277,9 +1397,7 @@ ChatPage::decryptDownloadedSecrets(mtx::secret_storage::AesHmacSha2KeyDescriptio
         http::client()->keys_signatures_upload(
           req, [](const mtx::responses::KeySignaturesUpload &res, mtx::http::RequestErr err) {
               if (err) {
-                  nhlog::net()->error("failed to upload signatures: {},{}",
-                                      mtx::errors::to_string(err->matrix_error.errcode),
-                                      static_cast<int>(err->status_code));
+                  nhlog::net()->error("failed to upload signatures: {}", *err);
               }
 
               for (const auto &[user_id, tmp] : res.errors)
@@ -1513,6 +1631,23 @@ ChatPage::handleMatrixUri(QString uri)
     return false;
 }
 
+void
+ChatPage::sendNotificationReply(const QString &roomid, const QString &eventid, const QString &body)
+{
+    view_manager_->queueReply(roomid, eventid, body);
+    auto exWin = MainWindow::instance()->windowForRoom(roomid);
+    if (exWin) {
+        exWin->setVisible(true);
+        exWin->raise();
+        exWin->requestActivate();
+    } else {
+        view_manager_->rooms()->setCurrentRoom(roomid);
+        MainWindow::instance()->setVisible(true);
+        MainWindow::instance()->raise();
+        MainWindow::instance()->requestActivate();
+    }
+}
+
 bool
 ChatPage::handleMatrixUri(const QUrl &uri)
 {
@@ -1525,3 +1660,11 @@ ChatPage::isRoomActive(const QString &room_id)
     return QGuiApplication::focusWindow() && QGuiApplication::focusWindow()->isActive() &&
            MainWindow::instance()->windowForRoom(room_id) == QGuiApplication::focusWindow();
 }
+
+void
+ChatPage::removeAllNotifications()
+{
+#if defined(Q_OS_LINUX)
+    notificationsManager->closeAllNotifications();
+#endif
+}
