@@ -1,6 +1,7 @@
 // SPDX-FileCopyrightText: 2017 Konstantinos Sideris <siderisk@auth.gr>
 // SPDX-FileCopyrightText: 2021 Nheko Contributors
 // SPDX-FileCopyrightText: 2022 Nheko Contributors
+// SPDX-FileCopyrightText: 2023 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -14,6 +15,7 @@
 #include <mtx/events.hpp>
 #include <mtx/events/encrypted.hpp>
 #include <mtx/events/member.hpp>
+#include <mtx/events/policy_rules.hpp>
 #include <mtx/events/presence.hpp>
 #include <mtx/secret_storage.hpp>
 
@@ -32,10 +34,6 @@ class NotificationsManager;
 class TimelineModel;
 class CallManager;
 
-constexpr int CONSENSUS_TIMEOUT      = 1000;
-constexpr int SHOW_CONTENT_TIMEOUT   = 3000;
-constexpr int TYPING_REFRESH_TIMEOUT = 10000;
-
 namespace mtx::requests {
 struct CreateRoom;
 }
@@ -48,7 +46,7 @@ struct Rooms;
 
 using SecretsToDecrypt = std::map<std::string, mtx::secret_storage::AesHmacSha2EncryptedData>;
 
-class ChatPage : public QObject
+class ChatPage final : public QObject
 {
     Q_OBJECT
 
@@ -74,6 +72,16 @@ public:
 
     void startChat(QString userid, std::optional<bool> encryptionEnabled);
 
+    //! Check if the given room is currently open.
+    bool isRoomActive(const QString &room_id);
+
+    const std::unique_ptr<mtx::pushrules::PushRuleEvaluator> &pushruleEvaluator() const
+    {
+        return pushrules;
+    }
+
+    void removeAllNotifications();
+
 public slots:
     bool handleMatrixUri(QString uri);
     bool handleMatrixUri(const QUrl &uri);
@@ -101,13 +109,11 @@ public slots:
     void receivedSessionKey(const std::string &room_id, const std::string &session_id);
     void decryptDownloadedSecrets(mtx::secret_storage::AesHmacSha2KeyDescription keyDesc,
                                   const SecretsToDecrypt &secrets);
+    void sendNotificationReply(const QString &roomid, const QString &eventid, const QString &body);
 signals:
     void connectionLost();
     void connectionRestored();
 
-    void notificationsRetrieved(const mtx::responses::Notifications &);
-    void highlightedNotifsRetrieved(const mtx::responses::Notifications &, const QPoint widgetPos);
-
     void contentLoaded();
     void closing();
     void changeWindowTitle(const int);
@@ -132,7 +138,6 @@ signals:
 
     void initializeViews(const mtx::responses::Sync &rooms);
     void initializeEmptyViews();
-    void initializeMentions(const QMap<QString, mtx::responses::Notifications> &notifs);
     void syncUI(const mtx::responses::Sync &sync);
     void dropToLoginPageCb(const QString &msg);
 
@@ -171,6 +176,7 @@ signals:
                        QString reason             = "",
                        bool failedJoin            = false,
                        bool promptForConfirmation = true);
+    void newOnlineKeyBackupAvailable();
 
 private slots:
     void logout();
@@ -193,9 +199,6 @@ private:
     void getProfileInfo();
     void getBackupVersion();
 
-    //! Check if the given room is currently open.
-    bool isRoomActive(const QString &room_id);
-
     using UserID      = QString;
     using Membership  = mtx::events::StateEvent<mtx::events::state::Member>;
     using Memberships = std::map<std::string, Membership>;
@@ -206,9 +209,6 @@ private:
     template<class Collection>
     Memberships getMemberships(const std::vector<Collection> &events) const;
 
-    //! Send desktop notification for the received messages.
-    void sendNotifications(const mtx::responses::Notifications &);
-
     template<typename T>
     void connectCallMessage();
 
@@ -222,6 +222,8 @@ private:
 
     NotificationsManager *notificationsManager;
     CallManager *callManager_;
+
+    std::unique_ptr<mtx::pushrules::PushRuleEvaluator> pushrules;
 };
 
 template<class Collection>
