@@ -1,16 +1,18 @@
 // SPDX-FileCopyrightText: 2021 Nheko Contributors
 // SPDX-FileCopyrightText: 2022 Nheko Contributors
+// SPDX-FileCopyrightText: 2023 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
 #include <QQuickImageProvider>
 
-class ColorImageProvider : public QQuickImageProvider
+class ColorImageProvider final : public QQuickImageProvider
 {
 public:
     ColorImageProvider()
       : QQuickImageProvider(QQuickImageProvider::Pixmap)
-    {}
+    {
+    }
 
     QPixmap requestPixmap(const QString &id, QSize *size, const QSize &requestedSize) override;
 };
