@@ -1,5 +1,6 @@
 // SPDX-FileCopyrightText: 2021 Nheko Contributors
 // SPDX-FileCopyrightText: 2022 Nheko Contributors
+// SPDX-FileCopyrightText: 2023 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -10,7 +11,7 @@
 #include <QSharedPointer>
 
 class SingleImagePackModel;
-class ImagePackListModel : public QAbstractListModel
+class ImagePackListModel final : public QAbstractListModel
 {
     Q_OBJECT
     Q_PROPERTY(bool containsAccountPack READ containsAccountPack CONSTANT)
