@@ -1,5 +1,6 @@
 // SPDX-FileCopyrightText: 2021 Nheko Contributors
 // SPDX-FileCopyrightText: 2022 Nheko Contributors
+// SPDX-FileCopyrightText: 2023 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -9,12 +10,15 @@
 #include <QAbstractListModel>
 #include <QVector>
 
-class Invitee : public QObject
+class Invitee final : public QObject
 {
     Q_OBJECT
 
 public:
-    Invitee(QString mxid, QObject *parent = nullptr);
+    Invitee(QString mxid,
+            QString displayName = "",
+            QString avatarUrl   = "",
+            QObject *parent     = nullptr);
 
 signals:
     void userInfoLoaded();
@@ -27,7 +31,7 @@ private:
     friend class InviteesModel;
 };
 
-class InviteesModel : public QAbstractListModel
+class InviteesModel final : public QAbstractListModel
 {
     Q_OBJECT
 
@@ -43,7 +47,7 @@ public:
 
     InviteesModel(QObject *parent = nullptr);
 
-    Q_INVOKABLE void addUser(QString mxid);
+    Q_INVOKABLE void addUser(QString mxid, QString displayName = "", QString avatarUrl = "");
     Q_INVOKABLE void removeUser(QString mxid);
 
     [[nodiscard]] QHash<int, QByteArray> roleNames() const override;
