@@ -1,5 +1,6 @@
 // SPDX-FileCopyrightText: 2021 Nheko Contributors
 // SPDX-FileCopyrightText: 2022 Nheko Contributors
+// SPDX-FileCopyrightText: 2023 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -14,7 +15,7 @@
 
 #include "jdenticoninterface.h"
 
-class JdenticonRunnable
+class JdenticonRunnable final
   : public QObject
   , public QRunnable
 {
@@ -34,7 +35,7 @@ private:
     QSize m_requestedSize;
 };
 
-class JdenticonResponse : public QQuickImageResponse
+class JdenticonResponse final : public QQuickImageResponse
 {
 public:
     JdenticonResponse(const QString &key, bool crop, double radius, const QSize &requestedSize);
