@@ -1,5 +1,6 @@
 // SPDX-FileCopyrightText: 2021 Nheko Contributors
 // SPDX-FileCopyrightText: 2022 Nheko Contributors
+// SPDX-FileCopyrightText: 2023 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -31,7 +32,7 @@ MemberListBackend::MemberListBackend(const QString &room_id, QObject *parent)
         // HACK: due to QTBUG-1020169, we'll load a big chunk to speed things up
         auto members = cache::getMembers(room_id_.toStdString(), 0, -1);
         addUsers(members);
-        numUsersLoaded_ = members.size();
+        numUsersLoaded_ = (int)members.size();
     } catch (const lmdb::error &e) {
         nhlog::db()->critical("Failed to retrieve members from cache: {}", e.what());
     }
@@ -40,13 +41,17 @@ MemberListBackend::MemberListBackend(const QString &room_id, QObject *parent)
 void
 MemberListBackend::addUsers(const std::vector<RoomMember> &members)
 {
-    beginInsertRows(QModelIndex{}, m_memberList.count(), m_memberList.count() + members.size() - 1);
+    auto thisRoom = ChatPage::instance()->timelineManager()->rooms()->getRoomById(room_id_);
+    if (thisRoom.isNull()) {
+        nhlog::ui()->error("Could not load the current room");
+        return;
+    }
+
+    beginInsertRows(
+      QModelIndex{}, m_memberList.count(), m_memberList.count() + (int)members.size() - 1);
 
     for (const auto &member : members)
-        m_memberList.push_back(
-          {member,
-           ChatPage::instance()->timelineManager()->rooms()->currentRoom()->avatarUrl(
-             member.user_id)});
+        m_memberList.push_back({member, thisRoom->avatarUrl(member.user_id)});
 
     endInsertRows();
 }
@@ -113,7 +118,7 @@ MemberListBackend::fetchMore(const QModelIndex &)
 
     auto members = cache::getMembers(room_id_.toStdString(), rowCount());
     addUsers(members);
-    numUsersLoaded_ += members.size();
+    numUsersLoaded_ += (int)members.size();
     emit numUsersLoadedChanged();
 
     loadingMoreMembers_ = false;
