@@ -1,5 +1,6 @@
 // SPDX-FileCopyrightText: 2021 Nheko Contributors
 // SPDX-FileCopyrightText: 2022 Nheko Contributors
+// SPDX-FileCopyrightText: 2023 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -9,13 +10,12 @@
 #include <QQuickImageResponse>
 
 #include <QImage>
-//#include <QThreadPool>
 
 #include <functional>
 
 #include <mtx/common.hpp>
 
-class MxcImageRunnable : public QObject
+class MxcImageRunnable final : public QObject
 {
     Q_OBJECT
 
@@ -29,7 +29,8 @@ public:
       , m_requestedSize(requestedSize)
       , m_crop(crop)
       , m_radius(radius)
-    {}
+    {
+    }
 
     void run();
 
@@ -38,7 +39,7 @@ public:
     bool m_crop;
     double m_radius;
 };
-class MxcImageResponse : public QQuickImageResponse
+class MxcImageResponse final : public QQuickImageResponse
 {
 public:
     MxcImageResponse(const QString &id, bool crop, double radius, const QSize &requestedSize)
@@ -80,6 +81,10 @@ class MxcImageProvider
   public QQuickAsyncImageProvider
 {
     Q_OBJECT
+
+public:
+    MxcImageProvider(QObject *parent = nullptr);
+
 public slots:
     QQuickImageResponse *
     requestImageResponse(const QString &id, const QSize &requestedSize) override;
