@@ -1,5 +1,6 @@
 // SPDX-FileCopyrightText: 2021 Nheko Contributors
 // SPDX-FileCopyrightText: 2022 Nheko Contributors
+// SPDX-FileCopyrightText: 2023 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -85,8 +86,9 @@ ReadReceiptsModel::addUsers(
     auto newReceipts = users.size() - readReceipts_.size();
 
     if (newReceipts > 0) {
-        beginInsertRows(
-          QModelIndex{}, readReceipts_.size(), readReceipts_.size() + newReceipts - 1);
+        beginInsertRows(QModelIndex{},
+                        static_cast<int>(readReceipts_.size()),
+                        static_cast<int>(readReceipts_.size() + newReceipts - 1));
 
         for (const auto &user : users) {
             QPair<QString, QDateTime> item = {QString::fromStdString(user.second),
