@@ -1,5 +1,6 @@
 // SPDX-FileCopyrightText: 2021 Nheko Contributors
 // SPDX-FileCopyrightText: 2022 Nheko Contributors
+// SPDX-FileCopyrightText: 2023 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -12,7 +13,7 @@
 #include <QSortFilterProxyModel>
 #include <QString>
 
-class ReadReceiptsModel : public QAbstractListModel
+class ReadReceiptsModel final : public QAbstractListModel
 {
     Q_OBJECT
 
@@ -51,7 +52,7 @@ private:
     QVector<QPair<QString, QDateTime>> readReceipts_;
 };
 
-class ReadReceiptsProxy : public QSortFilterProxyModel
+class ReadReceiptsProxy final : public QSortFilterProxyModel
 {
     Q_OBJECT
 
