@@ -1,6 +1,7 @@
 // SPDX-FileCopyrightText: 2017 Konstantinos Sideris <siderisk@auth.gr>
 // SPDX-FileCopyrightText: 2021 Nheko Contributors
 // SPDX-FileCopyrightText: 2022 Nheko Contributors
+// SPDX-FileCopyrightText: 2023 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -27,7 +28,7 @@ RegisterPage::RegisterPage(QObject *parent)
 }
 
 void
-RegisterPage::setError(QString err)
+RegisterPage::setError(const QString &err)
 {
     registrationError_ = err;
     emit errorChanged();
@@ -35,7 +36,7 @@ RegisterPage::setError(QString err)
     emit registeringChanged();
 }
 void
-RegisterPage::setHsError(QString err)
+RegisterPage::setHsError(const QString &err)
 {
     hsError_ = err;
     emit hsErrorChanged();
@@ -50,7 +51,7 @@ RegisterPage::initialDeviceName() const
 }
 
 void
-RegisterPage::setServer(QString server)
+RegisterPage::setServer(const QString &server)
 {
     if (server == lastServer)
         return;
@@ -67,7 +68,11 @@ RegisterPage::setServer(QString server)
     emit lookingUpHsChanged();
 
     http::client()->well_known(
-      [this](const mtx::responses::WellKnown &res, mtx::http::RequestErr err) {
+      [this, prevServer = server](const mtx::responses::WellKnown &res, mtx::http::RequestErr err) {
+          // server changed in between
+          if (lastServer != prevServer)
+              return;
+
           if (err) {
               if (err->status_code == 404) {
                   nhlog::net()->info("Autodiscovery: No .well-known.");
@@ -78,16 +83,16 @@ RegisterPage::setServer(QString server)
 
               if (!err->parse_error.empty()) {
                   setHsError(tr("Autodiscovery failed. Received malformed response."));
-                  nhlog::net()->error("Autodiscovery failed. Received malformed response.");
+                  nhlog::net()->error("Autodiscovery failed. Received malformed response. {}",
+                                      err->parse_error);
                   emit hsErrorChanged();
                   return;
               }
 
               setHsError(tr("Autodiscovery failed. Unknown error when requesting .well-known."));
               nhlog::net()->error("Autodiscovery failed. Unknown error when "
-                                  "requesting .well-known. {} {}",
-                                  err->status_code,
-                                  err->error_code);
+                                  "requesting .well-known. {}",
+                                  *err);
               return;
           }
 
@@ -104,68 +109,73 @@ RegisterPage::versionsCheck()
 {
     // Make a request to /_matrix/client/versions to check the address
     // given is a Matrix homeserver.
-    http::client()->versions(
-      [this](const mtx::responses::Versions &versions, mtx::http::RequestErr err) {
-          if (err) {
-              if (err->status_code == 404) {
-                  setHsError(
-                    tr("The required endpoints were not found. Possibly not a Matrix server."));
-                  emit hsErrorChanged();
-                  return;
-              }
+    http::client()->versions([this](const mtx::responses::Versions &versions,
+                                    mtx::http::RequestErr err) {
+        if (err) {
+            if (err->status_code == 404) {
+                setHsError(
+                  tr("The required endpoints were not found. Possibly not a Matrix server."));
+                emit hsErrorChanged();
+                return;
+            }
 
-              if (!err->parse_error.empty()) {
-                  setHsError(
-                    tr("Received malformed response. Make sure the homeserver domain is valid."));
-                  emit hsErrorChanged();
-                  return;
-              }
+            if (!err->parse_error.empty()) {
+                setHsError(
+                  tr("Received malformed response. Make sure the homeserver domain is valid."));
+                emit hsErrorChanged();
+                return;
+            }
 
-              setHsError(tr("An unknown error occured. Make sure the homeserver domain is valid."));
-              emit hsErrorChanged();
-              return;
-          }
+            setHsError(tr("An unknown error occured. Make sure the homeserver domain is valid."));
+            emit hsErrorChanged();
+            return;
+        }
 
-          if (std::find_if(
-                versions.versions.cbegin(), versions.versions.cend(), [](const std::string &v) {
-                    static const std::set<std::string_view, std::less<>> supported{
-                      "v1.1",
-                      "v1.2",
-                      "v1.3",
-                    };
-                    return supported.count(v) != 0;
-                }) == versions.versions.cend()) {
-              emit setHsError(
-                tr("The selected server does not support a version of the Matrix protocol, that "
-                   "this client understands (v1.1, v1.2 or v1.3). You can't register."));
-              emit hsErrorChanged();
-              return;
-          }
+        if (std::find_if(
+              versions.versions.cbegin(), versions.versions.cend(), [](const std::string &v) {
+                  static const std::set<std::string_view, std::less<>> supported{
+                    "v1.1",
+                    "v1.2",
+                    "v1.3",
+                    "v1.4",
+                    "v1.5",
+                  };
+                  return supported.count(v) != 0;
+              }) == versions.versions.cend()) {
+            emit setHsError(
+              tr("The selected server does not support a version of the Matrix protocol that "
+                 "this client understands (v1.1 to v1.5). You can't register."));
+            emit hsErrorChanged();
+            return;
+        }
 
-          http::client()->registration(
-            [this](const mtx::responses::Register &, mtx::http::RequestErr e) {
-                nhlog::net()->debug("Registration check: {}", e);
+        http::client()->registration([this](const mtx::responses::Register &,
+                                            mtx::http::RequestErr e) {
+            nhlog::net()->debug("Registration check: {}", e);
 
-                if (!e) {
-                    setHsError(tr("Server does not support querying registration flows!"));
-                    emit hsErrorChanged();
-                    return;
-                }
-                if (e->status_code != 401) {
-                    setHsError(tr("Server does not support registration."));
-                    emit hsErrorChanged();
-                    return;
-                }
+            if (!e) {
+                setHsError(tr("Server does not support querying registration flows!"));
+                emit hsErrorChanged();
+                return;
+            }
+            if (e->status_code != 401) {
+                setHsError(tr("Server does not support registration."));
+                emit hsErrorChanged();
+                return;
+            }
 
-                supported_   = true;
-                lookingUpHs_ = false;
-                emit lookingUpHsChanged();
-            });
-      });
+            for (const auto &f : e->matrix_error.unauthorized.flows)
+                nhlog::ui()->debug("Registration flows for server: {}", fmt::join(f.stages, ", "));
+
+            supported_   = true;
+            lookingUpHs_ = false;
+            emit lookingUpHsChanged();
+        });
+    });
 }
 
 void
-RegisterPage::checkUsername(QString name)
+RegisterPage::checkUsername(const QString &name)
 {
     usernameAvailable_ = usernameUnavailable_ = false;
     usernameError_.clear();
@@ -197,7 +207,9 @@ RegisterPage::checkUsername(QString name)
 }
 
 void
-RegisterPage::startRegistration(QString username, QString password, QString devicename)
+RegisterPage::startRegistration(const QString &username,
+                                const QString &password,
+                                const QString &devicename)
 {
     // These inputs should still be alright, but check just in case
     if (!username.isEmpty() && !password.isEmpty() && usernameAvailable_ && supported_) {
@@ -206,7 +218,7 @@ RegisterPage::startRegistration(QString username, QString password, QString devi
         registering_ = true;
         emit registeringChanged();
 
-        connect(UIA::instance(), &UIA::error, this, [this](QString msg) {
+        connect(UIA::instance(), &UIA::error, this, [this](const QString &msg) {
             setError(msg);
             disconnect(UIA::instance(), &UIA::error, this, nullptr);
         });
@@ -228,18 +240,13 @@ RegisterPage::startRegistration(QString username, QString password, QString devi
 
               // The server requires registration flows.
               if (err->status_code == 401 && err->matrix_error.unauthorized.flows.empty()) {
-                  nhlog::net()->warn("failed to retrieve registration flows: "
-                                     "status_code({}), matrix_error({}) ",
-                                     static_cast<int>(err->status_code),
-                                     err->matrix_error.error);
+                  nhlog::net()->warn("failed to retrieve registration flows: {}", *err);
                   setError(QString::fromStdString(err->matrix_error.error));
                   disconnect(UIA::instance(), &UIA::error, this, nullptr);
                   return;
               }
 
-              nhlog::net()->error("failed to register: status_code ({}), matrix_error({})",
-                                  static_cast<int>(err->status_code),
-                                  err->matrix_error.error);
+              nhlog::net()->error("failed to register: {}", *err);
 
               setError(QString::fromStdString(err->matrix_error.error));
               disconnect(UIA::instance(), &UIA::error, this, nullptr);
