@@ -1,16 +1,17 @@
 // SPDX-FileCopyrightText: 2021 Nheko Contributors
 // SPDX-FileCopyrightText: 2022 Nheko Contributors
+// SPDX-FileCopyrightText: 2023 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
 #pragma once
 
-#include "Cache.h"
+#include "CacheStructs.h"
 
 #include <QAbstractListModel>
 #include <QString>
 
-class RoomsModel : public QAbstractListModel
+class RoomsModel final : public QAbstractListModel
 {
 public:
     enum Roles
@@ -26,13 +27,11 @@ public:
     int rowCount(const QModelIndex &parent = QModelIndex()) const override
     {
         (void)parent;
-        return (int)roomids.size();
+        return (int)rooms.size();
     }
     QVariant data(const QModelIndex &index, int role) const override;
 
 private:
-    std::vector<QString> roomids;
-    std::vector<QString> roomAliases;
-    std::map<QString, RoomInfo> roomInfos;
+    std::vector<RoomNameAlias> rooms;
     bool showOnlyRoomWithAliases_;
 };
