@@ -1,5 +1,6 @@
 // SPDX-FileCopyrightText: 2021 Nheko Contributors
 // SPDX-FileCopyrightText: 2022 Nheko Contributors
+// SPDX-FileCopyrightText: 2023 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -8,7 +9,7 @@
 #include <QObject>
 #include <string>
 
-class SSOHandler : public QObject
+class SSOHandler final : public QObject
 {
     Q_OBJECT
 
