@@ -1,5 +1,6 @@
 // SPDX-FileCopyrightText: 2021 Nheko Contributors
 // SPDX-FileCopyrightText: 2022 Nheko Contributors
+// SPDX-FileCopyrightText: 2023 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -248,6 +249,9 @@ SingleImagePackModel::setIsStickerPack(bool val)
     using mtx::events::msc2545::PackUsage;
     if (val != pack.pack->is_sticker()) {
         pack.pack->usage.set(PackUsage::Sticker, val);
+        if (!val)
+            pack.pack->usage.set(PackUsage::Emoji, true);
+        emit isEmotePackChanged();
         emit isStickerPackChanged();
     }
 }
@@ -258,7 +262,10 @@ SingleImagePackModel::setIsEmotePack(bool val)
     using mtx::events::msc2545::PackUsage;
     if (val != pack.pack->is_emoji()) {
         pack.pack->usage.set(PackUsage::Emoji, val);
+        if (!val)
+            pack.pack->usage.set(PackUsage::Sticker, true);
         emit isEmotePackChanged();
+        emit isStickerPackChanged();
     }
 }
 
