@@ -1,6 +1,7 @@
 // SPDX-FileCopyrightText: 2017 Konstantinos Sideris <siderisk@auth.gr>
 // SPDX-FileCopyrightText: 2021 Nheko Contributors
 // SPDX-FileCopyrightText: 2022 Nheko Contributors
+// SPDX-FileCopyrightText: 2023 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -14,7 +15,7 @@
 class QAction;
 class QPainter;
 
-class MsgCountComposedIcon : public QIconEngine
+class MsgCountComposedIcon final : public QIconEngine
 {
 public:
     MsgCountComposedIcon(const QString &filename);
@@ -36,7 +37,7 @@ private:
     QIcon icon_;
 };
 
-class TrayIcon : public QSystemTrayIcon
+class TrayIcon final : public QSystemTrayIcon
 {
     Q_OBJECT
 public:
