@@ -1,6 +1,7 @@
 // SPDX-FileCopyrightText: 2017 Konstantinos Sideris <siderisk@auth.gr>
 // SPDX-FileCopyrightText: 2021 Nheko Contributors
 // SPDX-FileCopyrightText: 2022 Nheko Contributors
+// SPDX-FileCopyrightText: 2023 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -63,16 +64,19 @@ UserSettings::load(std::optional<QString> profile)
       settings.value(QStringLiteral("user/desktop_notifications"), true).toBool();
     hasAlertOnNotification_ =
       settings.value(QStringLiteral("user/alert_on_notification"), false).toBool();
-    groupView_         = settings.value(QStringLiteral("user/group_view"), true).toBool();
+    groupView_ = settings.value(QStringLiteral("user/group_view"), true).toBool();
+    scrollbarsInRoomlist_ =
+      settings.value(QStringLiteral("user/scrollbars_in_roomlist"), false).toBool();
     buttonsInTimeline_ = settings.value(QStringLiteral("user/timeline/buttons"), true).toBool();
     timelineMaxWidth_  = settings.value(QStringLiteral("user/timeline/max_width"), 0).toInt();
     messageHoverHighlight_ =
       settings.value(QStringLiteral("user/timeline/message_hover_highlight"), false).toBool();
     enlargeEmojiOnlyMessages_ =
       settings.value(QStringLiteral("user/timeline/enlarge_emoji_only_msg"), false).toBool();
-    markdown_     = settings.value(QStringLiteral("user/markdown_enabled"), true).toBool();
-    bubbles_      = settings.value(QStringLiteral("user/bubbles_enabled"), false).toBool();
-    smallAvatars_ = settings.value(QStringLiteral("user/small_avatars_enabled"), false).toBool();
+    markdown_       = settings.value(QStringLiteral("user/markdown_enabled"), true).toBool();
+    invertEnterKey_ = settings.value(QStringLiteral("user/invert_enter_key"), false).toBool();
+    bubbles_        = settings.value(QStringLiteral("user/bubbles_enabled"), false).toBool();
+    smallAvatars_   = settings.value(QStringLiteral("user/small_avatars_enabled"), false).toBool();
     animateImagesOnHover_ =
       settings.value(QStringLiteral("user/animate_images_on_hover"), false).toBool();
     typingNotifications_ =
@@ -88,7 +92,11 @@ UserSettings::load(std::optional<QString> profile)
     openImageExternal_ = settings.value(QStringLiteral("user/open_image_external"), false).toBool();
     openVideoExternal_ = settings.value(QStringLiteral("user/open_video_external"), false).toBool();
     decryptSidebar_    = settings.value(QStringLiteral("user/decrypt_sidebar"), true).toBool();
+    decryptNotifications_ =
+      settings.value(QStringLiteral("user/decrypt_notifications"), true).toBool();
     spaceNotifications_ = settings.value(QStringLiteral("user/space_notifications"), true).toBool();
+    fancyEffects_       = settings.value(QStringLiteral("user/fancy_effects"), true).toBool();
+    reducedMotion_      = settings.value(QStringLiteral("user/reduced_motion"), false).toBool();
     privacyScreen_      = settings.value(QStringLiteral("user/privacy_screen"), false).toBool();
     privacyScreenTimeout_ =
       settings.value(QStringLiteral("user/privacy_screen_timeout"), 0).toInt();
@@ -221,6 +229,17 @@ UserSettings::setGroupView(bool state)
     save();
 }
 
+void
+UserSettings::setScrollbarsInRoomlist(bool state)
+{
+    if (scrollbarsInRoomlist_ == state)
+        return;
+
+    scrollbarsInRoomlist_ = state;
+    emit scrollbarsInRoomlistChanged(state);
+    save();
+}
+
 void
 UserSettings::setHiddenTags(const QStringList &hiddenTags)
 {
@@ -287,6 +306,17 @@ UserSettings::setMarkdown(bool state)
     save();
 }
 
+void
+UserSettings::setInvertEnterKey(bool state)
+{
+    if (state == invertEnterKey_)
+        return;
+
+    invertEnterKey_ = state;
+    emit invertEnterKeyChanged(state);
+    save();
+}
+
 void
 UserSettings::setBubbles(bool state)
 {
@@ -425,6 +455,16 @@ UserSettings::setDecryptSidebar(bool state)
     save();
 }
 
+void
+UserSettings::setDecryptNotifications(bool state)
+{
+    if (state == decryptNotifications_)
+        return;
+    decryptNotifications_ = state;
+    emit decryptNotificationsChanged(state);
+    save();
+}
+
 void
 UserSettings::setSpaceNotifications(bool state)
 {
@@ -435,6 +475,32 @@ UserSettings::setSpaceNotifications(bool state)
     save();
 }
 
+void
+UserSettings::setFancyEffects(bool state)
+{
+    if (state == fancyEffects_)
+        return;
+    fancyEffects_ = state;
+    emit fancyEffectsChanged(state);
+    save();
+}
+
+void
+UserSettings::setReducedMotion(bool state)
+{
+    if (state == reducedMotion_)
+        return;
+    reducedMotion_ = state;
+    emit reducedMotionChanged(state);
+    save();
+
+    // Also toggle other motion related settings
+    if (reducedMotion_) {
+        setFancyEffects(false);
+        setAnimateImagesOnHover(true);
+    }
+}
+
 void
 UserSettings::setPrivacyScreen(bool state)
 {
@@ -796,7 +862,10 @@ UserSettings::save()
 
     settings.setValue(QStringLiteral("avatar_circles"), avatarCircles_);
     settings.setValue(QStringLiteral("decrypt_sidebar"), decryptSidebar_);
+    settings.setValue(QStringLiteral("decrypt_notificatons"), decryptNotifications_);
     settings.setValue(QStringLiteral("space_notifications"), spaceNotifications_);
+    settings.setValue(QStringLiteral("fancy_effects"), fancyEffects_);
+    settings.setValue(QStringLiteral("reduced_motion"), reducedMotion_);
     settings.setValue(QStringLiteral("privacy_screen"), privacyScreen_);
     settings.setValue(QStringLiteral("privacy_screen_timeout"), privacyScreenTimeout_);
     settings.setValue(QStringLiteral("mobile_mode"), mobileMode_);
@@ -806,7 +875,9 @@ UserSettings::save()
     settings.setValue(QStringLiteral("minor_events"), sortByImportance_);
     settings.setValue(QStringLiteral("read_receipts"), readReceipts_);
     settings.setValue(QStringLiteral("group_view"), groupView_);
+    settings.setValue(QStringLiteral("scrollbars_in_roomlist"), scrollbarsInRoomlist_);
     settings.setValue(QStringLiteral("markdown_enabled"), markdown_);
+    settings.setValue(QStringLiteral("invert_enter_key"), invertEnterKey_);
     settings.setValue(QStringLiteral("bubbles_enabled"), bubbles_);
     settings.setValue(QStringLiteral("small_avatars_enabled"), smallAvatars_);
     settings.setValue(QStringLiteral("animate_images_on_hover"), animateImagesOnHover_);
@@ -911,9 +982,13 @@ UserSettingsModel::data(const QModelIndex &index, int role) const
         case StartInTray:
             return tr("Start in tray");
         case GroupView:
-            return tr("Groups sidebar");
+            return tr("Communities sidebar");
+        case ScrollbarsInRoomlist:
+            return tr("Scrollbars in room list");
         case Markdown:
             return tr("Send messages as Markdown");
+        case InvertEnterKey:
+            return tr("Use shift+enter to send and enter to start a new line");
         case Bubbles:
             return tr("Enable message bubbles");
         case SmallAvatars:
@@ -944,8 +1019,14 @@ UserSettingsModel::data(const QModelIndex &index, int role) const
             return tr("Open videos with external program");
         case DecryptSidebar:
             return tr("Decrypt messages in sidebar");
+        case DecryptNotifications:
+            return tr("Decrypt notifications");
         case SpaceNotifications:
-            return tr("Show message counts for spaces");
+            return tr("Show message counts for communities and tags");
+        case FancyEffects:
+            return tr("Display fancy effects such as confetti");
+        case ReducedMotion:
+            return tr("Reduce or disable animations");
         case PrivacyScreen:
             return tr("Privacy Screen");
         case PrivacyScreenTimeout:
@@ -994,6 +1075,8 @@ UserSettingsModel::data(const QModelIndex &index, int role) const
             return tr("Platform");
         case GeneralSection:
             return tr("GENERAL");
+        case AccessibilitySection:
+            return tr("ACCESSIBILITY");
         case TimelineSection:
             return tr("TIMELINE");
         case SidebarSection:
@@ -1044,8 +1127,12 @@ UserSettingsModel::data(const QModelIndex &index, int role) const
             return i->startInTray();
         case GroupView:
             return i->groupView();
+        case ScrollbarsInRoomlist:
+            return i->scrollbarsInRoomlist();
         case Markdown:
             return i->markdown();
+        case InvertEnterKey:
+            return i->invertEnterKey();
         case Bubbles:
             return i->bubbles();
         case SmallAvatars:
@@ -1076,8 +1163,14 @@ UserSettingsModel::data(const QModelIndex &index, int role) const
             return i->openVideoExternal();
         case DecryptSidebar:
             return i->decryptSidebar();
+        case DecryptNotifications:
+            return i->decryptNotifications();
         case SpaceNotifications:
             return i->spaceNotifications();
+        case FancyEffects:
+            return i->fancyEffects();
+        case ReducedMotion:
+            return i->reducedMotion();
         case PrivacyScreen:
             return i->privacyScreen();
         case PrivacyScreenTimeout:
@@ -1161,7 +1254,7 @@ UserSettingsModel::data(const QModelIndex &index, int role) const
             return {};
         case TimelineMaxWidth:
             return tr("Set the max width of messages in the timeline (in pixels). This can help "
-                      "readability on wide screen, when Nheko is maximised");
+                      "readability on wide screen when Nheko is maximized");
         case PrivacyScreenTimeout:
             return tr(
               "Set timeout (in seconds) for how long after window loses\nfocus before the screen"
@@ -1179,11 +1272,17 @@ UserSettingsModel::data(const QModelIndex &index, int role) const
         case StartInTray:
             return tr("Start the application in the background without showing the client window.");
         case GroupView:
-            return tr("Show a column containing groups and tags next to the room list.");
+            return tr("Show a column containing communities and tags next to the room list.");
+        case ScrollbarsInRoomlist:
+            return tr("Shows scrollbars in the room list and communities list.");
         case Markdown:
             return tr(
               "Allow using markdown in messages.\nWhen disabled, all messages are sent as a plain "
               "text.");
+        case InvertEnterKey:
+            return tr(
+              "Invert the behavior of the enter key in the text input, making it send the message "
+              "when shift+enter is pressed and starting a new line when enter is pressed.");
         case Bubbles:
             return tr(
               "Messages get a bubble background. This also triggers some layout changes (WIP).");
@@ -1201,7 +1300,7 @@ UserSettingsModel::data(const QModelIndex &index, int role) const
               "be sorted by the timestamp of the last message in a room.\nIf this is on, rooms "
               "which "
               "have active notifications (the small circle with a number in it) will be sorted on "
-              "top. Rooms, that you have muted, will still be sorted by timestamp, since you don't "
+              "top. Rooms that you have muted will still be sorted by timestamp, since you don't "
               "seem to consider them as important as the other rooms.");
         case ButtonsInTimeline:
             return tr(
@@ -1233,9 +1332,17 @@ UserSettingsModel::data(const QModelIndex &index, int role) const
         case DecryptSidebar:
             return tr("Decrypt the messages shown in the sidebar.\nOnly affects messages in "
                       "encrypted chats.");
+        case DecryptNotifications:
+            return tr("Decrypt messages shown in notifications for encrypted chats.");
         case SpaceNotifications:
-            return tr(
-              "Choose where to show the total number of notifications contained within a space.");
+            return tr("Choose where to show the total number of notifications contained within a "
+                      "community or tag.");
+        case FancyEffects:
+            return tr("Some messages can be sent with fancy effects. For example, messages sent "
+                      "with '/confetti' will show confetti on screen.");
+        case ReducedMotion:
+            return tr("Nheko uses animations in several places to make stuff pretty. This allows "
+                      "you to turn those off if they make you feel unwell.");
         case PrivacyScreen:
             return tr("When the window loses focus, the timeline will\nbe blurred.");
         case MobileMode:
@@ -1251,7 +1358,7 @@ UserSettingsModel::data(const QModelIndex &index, int role) const
                       "improves safety but makes E2EE more tedious.");
         case ShareKeysWithTrustedUsers:
             return tr(
-              "Automatically replies to key requests from other users, if they are verified, "
+              "Automatically replies to key requests from other users if they are verified, "
               "even if that device shouldn't have access to those keys otherwise.");
         case UseOnlineKeyBackup:
             return tr(
@@ -1266,6 +1373,7 @@ UserSettingsModel::data(const QModelIndex &index, int role) const
         case Version:
         case Platform:
         case GeneralSection:
+        case AccessibilitySection:
         case TimelineSection:
         case SidebarSection:
         case TraySection:
@@ -1323,7 +1431,9 @@ UserSettingsModel::data(const QModelIndex &index, int role) const
         case Tray:
         case StartInTray:
         case GroupView:
+        case ScrollbarsInRoomlist:
         case Markdown:
+        case InvertEnterKey:
         case Bubbles:
         case SmallAvatars:
         case AnimateImagesOnHover:
@@ -1338,6 +1448,7 @@ UserSettingsModel::data(const QModelIndex &index, int role) const
         case OpenImageExternal:
         case OpenVideoExternal:
         case DecryptSidebar:
+        case DecryptNotifications:
         case PrivacyScreen:
         case MobileMode:
         case UseStunServer:
@@ -1346,6 +1457,8 @@ UserSettingsModel::data(const QModelIndex &index, int role) const
         case UseOnlineKeyBackup:
         case ExposeDBusApi:
         case SpaceNotifications:
+        case FancyEffects:
+        case ReducedMotion:
             return Toggle;
         case Profile:
         case UserId:
@@ -1357,6 +1470,7 @@ UserSettingsModel::data(const QModelIndex &index, int role) const
         case Platform:
             return ReadOnlyText;
         case GeneralSection:
+        case AccessibilitySection:
         case TimelineSection:
         case SidebarSection:
         case TraySection:
@@ -1541,6 +1655,13 @@ UserSettingsModel::setData(const QModelIndex &index, const QVariant &value, int
             } else
                 return false;
         }
+        case ScrollbarsInRoomlist: {
+            if (value.userType() == QMetaType::Bool) {
+                i->setScrollbarsInRoomlist(value.toBool());
+                return true;
+            } else
+                return false;
+        }
         case Markdown: {
             if (value.userType() == QMetaType::Bool) {
                 i->setMarkdown(value.toBool());
@@ -1548,6 +1669,13 @@ UserSettingsModel::setData(const QModelIndex &index, const QVariant &value, int
             } else
                 return false;
         }
+        case InvertEnterKey: {
+            if (value.userType() == QMetaType::Bool) {
+                i->setInvertEnterKey(value.toBool());
+                return true;
+            } else
+                return false;
+        }
         case Bubbles: {
             if (value.userType() == QMetaType::Bool) {
                 i->setBubbles(value.toBool());
@@ -1653,7 +1781,13 @@ UserSettingsModel::setData(const QModelIndex &index, const QVariant &value, int
             } else
                 return false;
         }
-            return i->decryptSidebar();
+        case DecryptNotifications: {
+            if (value.userType() == QMetaType::Bool) {
+                i->setDecryptNotifications(value.toBool());
+                return true;
+            } else
+                return false;
+        }
         case SpaceNotifications: {
             if (value.userType() == QMetaType::Bool) {
                 i->setSpaceNotifications(value.toBool());
@@ -1661,6 +1795,20 @@ UserSettingsModel::setData(const QModelIndex &index, const QVariant &value, int
             } else
                 return false;
         }
+        case FancyEffects: {
+            if (value.userType() == QMetaType::Bool) {
+                i->setFancyEffects(value.toBool());
+                return true;
+            } else
+                return false;
+        }
+        case ReducedMotion: {
+            if (value.userType() == QMetaType::Bool) {
+                i->setReducedMotion(value.toBool());
+                return true;
+            } else
+                return false;
+        }
         case PrivacyScreen: {
             if (value.userType() == QMetaType::Bool) {
                 i->setPrivacyScreen(value.toBool());
@@ -1958,6 +2106,9 @@ UserSettingsModel::UserSettingsModel(QObject *p)
     connect(s.get(), &UserSettings::markdownChanged, this, [this]() {
         emit dataChanged(index(Markdown), index(Markdown), {Value});
     });
+    connect(s.get(), &UserSettings::invertEnterKeyChanged, this, [this]() {
+        emit dataChanged(index(InvertEnterKey), index(InvertEnterKey), {Value});
+    });
     connect(s.get(), &UserSettings::bubblesChanged, this, [this]() {
         emit dataChanged(index(Bubbles), index(Bubbles), {Value});
     });
@@ -1967,15 +2118,27 @@ UserSettingsModel::UserSettingsModel(QObject *p)
     connect(s.get(), &UserSettings::groupViewStateChanged, this, [this]() {
         emit dataChanged(index(GroupView), index(GroupView), {Value});
     });
+    connect(s.get(), &UserSettings::scrollbarsInRoomlistChanged, this, [this]() {
+        emit dataChanged(index(ScrollbarsInRoomlist), index(ScrollbarsInRoomlist), {Value});
+    });
     connect(s.get(), &UserSettings::roomSortingChanged, this, [this]() {
         emit dataChanged(index(SortByImportance), index(SortByImportance), {Value});
     });
     connect(s.get(), &UserSettings::decryptSidebarChanged, this, [this]() {
         emit dataChanged(index(DecryptSidebar), index(DecryptSidebar), {Value});
     });
-    connect(s.get(), &UserSettings::spaceNotificationsChanged, this, [this] {
+    connect(s.get(), &UserSettings::decryptNotificationsChanged, this, [this]() {
+        emit dataChanged(index(DecryptNotifications), index(DecryptNotifications), {Value});
+    });
+    connect(s.get(), &UserSettings::spaceNotificationsChanged, this, [this]() {
         emit dataChanged(index(SpaceNotifications), index(SpaceNotifications), {Value});
     });
+    connect(s.get(), &UserSettings::fancyEffectsChanged, this, [this]() {
+        emit dataChanged(index(FancyEffects), index(FancyEffects), {Value});
+    });
+    connect(s.get(), &UserSettings::reducedMotionChanged, this, [this]() {
+        emit dataChanged(index(ReducedMotion), index(ReducedMotion), {Value});
+    });
     connect(s.get(), &UserSettings::trayChanged, this, [this]() {
         emit dataChanged(index(Tray), index(Tray), {Value});
         emit dataChanged(index(StartInTray), index(StartInTray), {Enabled});
