@@ -23,10 +23,6 @@ class QSpinBox;
 class QHBoxLayout;
 class QVBoxLayout;
 
-constexpr int OptionMargin       = 6;
-constexpr int LayoutTopMargin    = 50;
-constexpr int LayoutBottomMargin = LayoutTopMargin;
-
 class UserSettings final : public QObject
 {
     Q_OBJECT
@@ -39,6 +35,8 @@ class UserSettings final : public QObject
     Q_PROPERTY(bool tray READ tray WRITE setTray NOTIFY trayChanged)
     Q_PROPERTY(bool startInTray READ startInTray WRITE setStartInTray NOTIFY startInTrayChanged)
     Q_PROPERTY(bool groupView READ groupView WRITE setGroupView NOTIFY groupViewStateChanged)
+    Q_PROPERTY(bool scrollbarsInRoomlist READ scrollbarsInRoomlist WRITE setScrollbarsInRoomlist
+                 NOTIFY scrollbarsInRoomlistChanged)
     Q_PROPERTY(bool markdown READ markdown WRITE setMarkdown NOTIFY markdownChanged)
     Q_PROPERTY(
       bool invertEnterKey READ invertEnterKey WRITE setInvertEnterKey NOTIFY invertEnterKeyChanged)
@@ -66,6 +64,8 @@ class UserSettings final : public QObject
     Q_PROPERTY(bool spaceNotifications READ spaceNotifications WRITE setSpaceNotifications NOTIFY
                  spaceNotificationsChanged)
     Q_PROPERTY(bool fancyEffects READ fancyEffects WRITE setFancyEffects NOTIFY fancyEffectsChanged)
+    Q_PROPERTY(
+      bool reducedMotion READ reducedMotion WRITE setReducedMotion NOTIFY reducedMotionChanged)
     Q_PROPERTY(
       bool privacyScreen READ privacyScreen WRITE setPrivacyScreen NOTIFY privacyScreenChanged)
     Q_PROPERTY(int privacyScreenTimeout READ privacyScreenTimeout WRITE setPrivacyScreenTimeout
@@ -155,6 +155,7 @@ public:
     void setFontFamily(QString family);
     void setEmojiFontFamily(QString family);
     void setGroupView(bool state);
+    void setScrollbarsInRoomlist(bool state);
     void setMarkdown(bool state);
     void setInvertEnterKey(bool state);
     void setBubbles(bool state);
@@ -174,6 +175,7 @@ public:
     void setDecryptNotifications(bool state);
     void setSpaceNotifications(bool state);
     void setFancyEffects(bool state);
+    void setReducedMotion(bool state);
     void setPrivacyScreen(bool state);
     void setPrivacyScreenTimeout(int state);
     void setPresence(Presence state);
@@ -213,11 +215,13 @@ public:
     bool tray() const { return tray_; }
     bool startInTray() const { return startInTray_; }
     bool groupView() const { return groupView_; }
+    bool scrollbarsInRoomlist() const { return scrollbarsInRoomlist_; }
     bool avatarCircles() const { return avatarCircles_; }
     bool decryptSidebar() const { return decryptSidebar_; }
     bool decryptNotifications() const { return decryptNotifications_; }
     bool spaceNotifications() const { return spaceNotifications_; }
     bool fancyEffects() const { return fancyEffects_; }
+    bool reducedMotion() const { return reducedMotion_; }
     bool privacyScreen() const { return privacyScreen_; }
     int privacyScreenTimeout() const { return privacyScreenTimeout_; }
     bool markdown() const { return markdown_; }
@@ -279,6 +283,7 @@ public:
 
 signals:
     void groupViewStateChanged(bool state);
+    void scrollbarsInRoomlistChanged(bool state);
     void roomSortingChanged(bool state);
     void themeChanged(QString state);
     void messageHoverHighlightChanged(bool state);
@@ -300,6 +305,7 @@ signals:
     void decryptNotificationsChanged(bool state);
     void spaceNotificationsChanged(bool state);
     void fancyEffectsChanged(bool state);
+    void reducedMotionChanged(bool state);
     void privacyScreenChanged(bool state);
     void privacyScreenTimeoutChanged(int state);
     void timelineMaxWidthChanged(int state);
@@ -350,6 +356,7 @@ private:
     bool tray_;
     bool startInTray_;
     bool groupView_;
+    bool scrollbarsInRoomlist_;
     bool markdown_;
     bool invertEnterKey_;
     bool bubbles_;
@@ -366,6 +373,7 @@ private:
     bool decryptNotifications_;
     bool spaceNotifications_;
     bool fancyEffects_;
+    bool reducedMotion_;
     bool privacyScreen_;
     int privacyScreenTimeout_;
     bool shareKeysWithTrustedUsers_;
@@ -430,15 +438,20 @@ class UserSettingsModel final : public QAbstractListModel
         UseIdenticon,
         PrivacyScreen,
         PrivacyScreenTimeout,
+        ScrollbarsInRoomlist,
 #ifdef NHEKO_DBUS_SYS
         ExposeDBusApi,
 #endif
 
+        AccessibilitySection,
+        ReducedMotion,
+        FancyEffects,
+        AnimateImagesOnHover,
+        MessageHoverHighlight,
+
         TimelineSection,
         TimelineMaxWidth,
-        MessageHoverHighlight,
         EnlargeEmojiOnlyMessages,
-        AnimateImagesOnHover,
         OpenImageExternal,
         OpenVideoExternal,
         ButtonsInTimeline,
@@ -448,7 +461,6 @@ class UserSettingsModel final : public QAbstractListModel
         InvertEnterKey,
         Bubbles,
         SmallAvatars,
-        FancyEffects,
 
         SidebarSection,
         GroupView,
