@@ -1,5 +1,6 @@
 // SPDX-FileCopyrightText: 2021 Nheko Contributors
 // SPDX-FileCopyrightText: 2022 Nheko Contributors
+// SPDX-FileCopyrightText: 2023 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -45,16 +46,17 @@ createDescriptionInfo(const Event &event, const QString &localUser, const QStrin
 
     const auto username = displayName;
     const auto ts       = QDateTime::fromMSecsSinceEpoch(msg.origin_server_ts);
-    auto body           = utils::event_body(event).trimmed();
+    auto body           = mtx::accessors::body(event);
     if (mtx::accessors::relations(event).reply_to())
-        body = QString::fromStdString(utils::stripReplyFromBody(body.toStdString()));
+        body = utils::stripReplyFromBody(body);
 
-    return DescInfo{QString::fromStdString(msg.event_id),
-                    sender,
-                    utils::messageDescription<T>(username, body, sender == localUser),
-                    utils::descriptiveTime(ts),
-                    msg.origin_server_ts,
-                    ts};
+    return DescInfo{
+      QString::fromStdString(msg.event_id),
+      sender,
+      utils::messageDescription<T>(username, QString::fromStdString(body), sender == localUser),
+      utils::descriptiveTime(ts),
+      msg.origin_server_ts,
+      ts};
 }
 
 std::string
@@ -131,8 +133,14 @@ utils::replaceEmoji(const QString &body)
     QVector<uint> utf32_string = body.toUcs4();
 
     bool insideFontBlock = false;
+    bool insideTag       = false;
     for (auto &code : utf32_string) {
-        if (utils::codepointIsEmoji(code)) {
+        if (code == U'<')
+            insideTag = true;
+        else if (code == U'>')
+            insideTag = false;
+
+        if (!insideTag && utils::codepointIsEmoji(code)) {
             if (!insideFontBlock) {
                 fmtBody += QStringLiteral("<font face=\"") % UserSettings::instance()->emojiFont() %
                            QStringLiteral("\">");
@@ -210,9 +218,11 @@ utils::getMessageDescription(const TimelineEvent &event,
     using Notice     = mtx::events::RoomEvent<mtx::events::msg::Notice>;
     using Text       = mtx::events::RoomEvent<mtx::events::msg::Text>;
     using Video      = mtx::events::RoomEvent<mtx::events::msg::Video>;
+    using Confetti   = mtx::events::RoomEvent<mtx::events::msg::Confetti>;
     using CallInvite = mtx::events::RoomEvent<mtx::events::voip::CallInvite>;
     using CallAnswer = mtx::events::RoomEvent<mtx::events::voip::CallAnswer>;
     using CallHangUp = mtx::events::RoomEvent<mtx::events::voip::CallHangUp>;
+    using CallReject = mtx::events::RoomEvent<mtx::events::voip::CallReject>;
     using Encrypted  = mtx::events::EncryptedEvent<mtx::events::msg::Encrypted>;
 
     if (std::holds_alternative<Audio>(event)) {
@@ -229,12 +239,16 @@ utils::getMessageDescription(const TimelineEvent &event,
         return createDescriptionInfo<Text>(event, localUser, displayName);
     } else if (std::holds_alternative<Video>(event)) {
         return createDescriptionInfo<Video>(event, localUser, displayName);
+    } else if (std::holds_alternative<Confetti>(event)) {
+        return createDescriptionInfo<Confetti>(event, localUser, displayName);
     } else if (std::holds_alternative<CallInvite>(event)) {
         return createDescriptionInfo<CallInvite>(event, localUser, displayName);
     } else if (std::holds_alternative<CallAnswer>(event)) {
         return createDescriptionInfo<CallAnswer>(event, localUser, displayName);
     } else if (std::holds_alternative<CallHangUp>(event)) {
         return createDescriptionInfo<CallHangUp>(event, localUser, displayName);
+    } else if (std::holds_alternative<CallReject>(event)) {
+        return createDescriptionInfo<CallReject>(event, localUser, displayName);
     } else if (std::holds_alternative<mtx::events::Sticker>(event)) {
         return createDescriptionInfo<mtx::events::Sticker>(event, localUser, displayName);
     } else if (auto msg = std::get_if<Encrypted>(&event); msg != nullptr) {
@@ -315,28 +329,6 @@ utils::levenshtein_distance(const std::string &s1, const std::string &s2)
     return *std::min_element(row1.begin(), row1.end());
 }
 
-QString
-utils::event_body(const mtx::events::collections::TimelineEvents &e)
-{
-    using namespace mtx::events;
-    if (auto ev = std::get_if<RoomEvent<msg::Audio>>(&e); ev != nullptr)
-        return QString::fromStdString(ev->content.body);
-    if (auto ev = std::get_if<RoomEvent<msg::Emote>>(&e); ev != nullptr)
-        return QString::fromStdString(ev->content.body);
-    if (auto ev = std::get_if<RoomEvent<msg::File>>(&e); ev != nullptr)
-        return QString::fromStdString(ev->content.body);
-    if (auto ev = std::get_if<RoomEvent<msg::Image>>(&e); ev != nullptr)
-        return QString::fromStdString(ev->content.body);
-    if (auto ev = std::get_if<RoomEvent<msg::Notice>>(&e); ev != nullptr)
-        return QString::fromStdString(ev->content.body);
-    if (auto ev = std::get_if<RoomEvent<msg::Text>>(&e); ev != nullptr)
-        return QString::fromStdString(ev->content.body);
-    if (auto ev = std::get_if<RoomEvent<msg::Video>>(&e); ev != nullptr)
-        return QString::fromStdString(ev->content.body);
-
-    return QString();
-}
-
 QPixmap
 utils::scaleImageToPixmap(const QImage &img, int size)
 {
@@ -345,8 +337,8 @@ utils::scaleImageToPixmap(const QImage &img, int size)
 
     // Deprecated in 5.13: const double sz =
     //  std::ceil(QApplication::desktop()->screen()->devicePixelRatioF() * (double)size);
-    const double sz =
-      std::ceil(QGuiApplication::primaryScreen()->devicePixelRatio() * (double)size);
+    const int sz = static_cast<int>(
+      std::ceil(QGuiApplication::primaryScreen()->devicePixelRatio() * (double)size));
     return QPixmap::fromImage(img.scaled(sz, sz, Qt::IgnoreAspectRatio, Qt::SmoothTransformation));
 }
 
@@ -367,8 +359,8 @@ utils::scaleDown(uint64_t maxWidth, uint64_t maxHeight, const QPixmap &source)
         w = source.width();
         h = source.height();
     } else {
-        w = source.width() * minAspectRatio;
-        h = source.height() * minAspectRatio;
+        w = static_cast<int>(static_cast<double>(source.width()) * minAspectRatio);
+        h = static_cast<int>(static_cast<double>(source.height()) * minAspectRatio);
     }
 
     return source.scaled(w, h, Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
@@ -443,7 +435,7 @@ utils::escapeBlacklistedHtml(const QString &rawStr)
     const auto end = data.cend();
     for (auto pos = data.cbegin(); pos < end;) {
         auto tagStart = std::find(pos, end, '<');
-        buffer.append(pos, tagStart - pos);
+        buffer.append(pos, static_cast<int>(tagStart - pos));
         if (tagStart == end)
             break;
 
@@ -451,21 +443,24 @@ utils::escapeBlacklistedHtml(const QString &rawStr)
         const auto tagNameEnd =
           std::find_first_of(tagNameStart, end, tagNameEnds.begin(), tagNameEnds.end());
 
-        if (allowedTags.find(QByteArray(tagNameStart, tagNameEnd - tagNameStart).toLower()) ==
+        if (allowedTags.find(
+              QByteArray(tagNameStart, static_cast<int>(tagNameEnd - tagNameStart)).toLower()) ==
             allowedTags.end()) {
             // not allowed -> escape
             buffer.append("&lt;");
             pos = tagNameStart;
             continue;
         } else {
-            buffer.append(tagStart, tagNameEnd - tagStart);
+            buffer.append(tagStart, static_cast<int>(tagNameEnd - tagStart));
 
             pos = tagNameEnd;
 
             if (tagNameEnd != end) {
                 auto attrStart = tagNameEnd;
                 auto attrsEnd  = std::find(attrStart, end, '>');
-                if (*(attrsEnd - 1) == '/')
+                // we don't want to consume the slash of self closing tags as part of an attribute.
+                // However, obviously we don't want to move backwards, if there are no attributes.
+                if (*(attrsEnd - 1) == '/' && attrStart < attrsEnd)
                     attrsEnd -= 1;
 
                 pos = attrsEnd;
@@ -483,7 +478,8 @@ utils::escapeBlacklistedHtml(const QString &rawStr)
                     auto attrEnd = std::find_first_of(
                       attrStart, attrsEnd, attrNameEnds.begin(), attrNameEnds.end());
 
-                    auto attrName = QByteArray(attrStart, attrEnd - attrStart).toLower();
+                    auto attrName =
+                      QByteArray(attrStart, static_cast<int>(attrEnd - attrStart)).toLower();
 
                     auto sanitizeValue = [&attrName](QByteArray val) {
                         if (attrName == QByteArrayLiteral("src") && !val.startsWith("mxc://"))
@@ -511,8 +507,8 @@ utils::escapeBlacklistedHtml(const QString &rawStr)
                                     if (valueEnd == attrsEnd)
                                         break;
 
-                                    auto val =
-                                      sanitizeValue(QByteArray(attrStart, valueEnd - attrStart));
+                                    auto val  = sanitizeValue(QByteArray(
+                                      attrStart, static_cast<int>(valueEnd - attrStart)));
                                     attrStart = consumeSpaces(valueEnd + 1);
                                     if (!val.isEmpty()) {
                                         buffer.append(' ');
@@ -528,8 +524,8 @@ utils::escapeBlacklistedHtml(const QString &rawStr)
                                     if (valueEnd == attrsEnd)
                                         break;
 
-                                    auto val =
-                                      sanitizeValue(QByteArray(attrStart, valueEnd - attrStart));
+                                    auto val  = sanitizeValue(QByteArray(
+                                      attrStart, static_cast<int>(valueEnd - attrStart)));
                                     attrStart = consumeSpaces(valueEnd + 1);
                                     if (!val.isEmpty()) {
                                         buffer.append(' ');
@@ -544,9 +540,9 @@ utils::escapeBlacklistedHtml(const QString &rawStr)
                                                                        attrsEnd,
                                                                        attrValueEnds.begin(),
                                                                        attrValueEnds.end());
-                                    auto val =
-                                      sanitizeValue(QByteArray(attrStart, valueEnd - attrStart));
-                                    attrStart = consumeSpaces(valueEnd);
+                                    auto val      = sanitizeValue(QByteArray(
+                                      attrStart, static_cast<int>(valueEnd - attrStart)));
+                                    attrStart     = consumeSpaces(valueEnd);
 
                                     if (val.contains('"'))
                                         continue;
@@ -572,96 +568,369 @@ utils::escapeBlacklistedHtml(const QString &rawStr)
     return QString::fromUtf8(buffer);
 }
 
+static void
+rainbowify(cmark_node *node)
+{
+    // create iterator over node
+    cmark_iter *iter = cmark_iter_new(node);
+
+    // First loop to get total text length
+    int textLen = 0;
+    while (cmark_iter_next(iter) != CMARK_EVENT_DONE) {
+        cmark_node *cur = cmark_iter_get_node(iter);
+        // only text nodes (no code or semilar)
+        if (cmark_node_get_type(cur) != CMARK_NODE_TEXT)
+            continue;
+        // count up by length of current node's text
+        QTextBoundaryFinder tbf(QTextBoundaryFinder::BoundaryType::Grapheme,
+                                QString(cmark_node_get_literal(cur)));
+        while (tbf.toNextBoundary() != -1)
+            textLen++;
+    }
+
+    // create new iter to start over
+    cmark_iter_free(iter);
+    iter = cmark_iter_new(node);
+
+    // Second loop to rainbowify
+    int charIdx = 0;
+    while (cmark_iter_next(iter) != CMARK_EVENT_DONE) {
+        cmark_node *cur = cmark_iter_get_node(iter);
+        // only text nodes (no code or similar)
+        if (cmark_node_get_type(cur) != CMARK_NODE_TEXT)
+            continue;
+
+        // get text in current node
+        QString nodeText(cmark_node_get_literal(cur));
+        // create buffer to append rainbow text to
+        QString buf;
+        int boundaryStart = 0;
+        int boundaryEnd   = 0;
+        // use QTextBoundaryFinder to iterate over graphemes
+        QTextBoundaryFinder tbf(QTextBoundaryFinder::BoundaryType::Grapheme, nodeText);
+        while ((boundaryEnd = tbf.toNextBoundary()) != -1) {
+            charIdx++;
+            // Split text to get current char
+            auto curChar  = QStringView(nodeText).mid(boundaryStart, boundaryEnd - boundaryStart);
+            boundaryStart = boundaryEnd;
+            // Don't rainbowify whitespaces
+            if (curChar.trimmed().isEmpty() || utils::codepointIsEmoji(curChar.toUcs4().at(0))) {
+                buf.append(curChar);
+                continue;
+            }
+
+            // get correct color for char index
+            // Use colors as described here:
+            // https://shark.comfsm.fm/~dleeling/cis/hsl_rainbow.html
+            auto color = QColor::fromHslF((charIdx - 1.0) / textLen * (5. / 6.), 0.9, 0.5);
+            // format color for HTML
+            auto colorString = color.name(QColor::NameFormat::HexRgb);
+            // create HTML element for current char
+            auto curCharColored =
+              QStringLiteral("<font color=\"%0\">%1</font>").arg(colorString).arg(curChar);
+            // append colored HTML element to buffer
+            buf.append(curCharColored);
+        }
+
+        // create HTML_INLINE node to prevent HTML from being escaped
+        auto htmlNode = cmark_node_new(CMARK_NODE_HTML_INLINE);
+        // set content of HTML node to buffer contents
+        cmark_node_set_literal(htmlNode, buf.toUtf8().data());
+        // replace current node with HTML node
+        cmark_node_replace(cur, htmlNode);
+        // free memory of old node
+        cmark_node_free(cur);
+    }
+
+    cmark_iter_free(iter);
+}
+
+static std::string
+extract_spoiler_warning(std::string &inside_spoiler)
+{
+    std::string spoiler_text;
+    if (auto spoilerTextEnd = inside_spoiler.find("|"); spoilerTextEnd != std::string::npos) {
+        spoiler_text   = inside_spoiler.substr(0, spoilerTextEnd);
+        inside_spoiler = inside_spoiler.substr(spoilerTextEnd + 1);
+    }
+    return QString::fromStdString(spoiler_text).replace('"', "&quot;").toStdString();
+}
+
+// TODO(Nico): Add tests :D
+static void
+process_spoilers(cmark_node *node)
+{
+    auto iter = cmark_iter_new(node);
+
+    while (cmark_iter_next(iter) != CMARK_EVENT_DONE) {
+        cmark_node *cur = cmark_iter_get_node(iter);
+
+        // only text nodes (no code or similar)
+        if (cmark_node_get_type(cur) != CMARK_NODE_TEXT) {
+            continue;
+        }
+
+        std::string_view content = cmark_node_get_literal(cur);
+
+        if (auto posStart = content.find("||"); posStart != std::string::npos) {
+            // we have the start of the spoiler
+            if (auto posEnd = content.find("||", posStart + 2); posEnd != std::string::npos) {
+                // we have the end of the spoiler in the same node
+
+                std::string before_spoiler = std::string(content.substr(0, posStart));
+                std::string inside_spoiler =
+                  std::string(content.substr(posStart + 2, posEnd - 2 - posStart));
+                std::string after_spoiler = std::string(content.substr(posEnd + 2));
+
+                std::string spoiler_text = extract_spoiler_warning(inside_spoiler);
+
+                // create the new nodes
+                auto before_node = cmark_node_new(cmark_node_type::CMARK_NODE_TEXT);
+                cmark_node_set_literal(before_node, before_spoiler.c_str());
+                auto after_node = cmark_node_new(cmark_node_type::CMARK_NODE_TEXT);
+                cmark_node_set_literal(after_node, after_spoiler.c_str());
+
+                auto block = cmark_node_new(cmark_node_type::CMARK_NODE_CUSTOM_INLINE);
+                cmark_node_set_on_enter(
+                  block, ("<span data-mx-spoiler=\"" + spoiler_text + "\">").c_str());
+                cmark_node_set_on_exit(block, "</span>");
+                auto child_node = cmark_node_new(cmark_node_type::CMARK_NODE_TEXT);
+                cmark_node_set_literal(child_node, inside_spoiler.c_str());
+                cmark_node_append_child(block, child_node);
+
+                // insert the new nodes into the tree
+                cmark_node_replace(cur, block);
+                cmark_node_insert_before(block, before_node);
+                cmark_node_insert_after(block, after_node);
+
+                // cleanup the replaced node
+                cmark_node_free(cur);
+
+                // fixup the iterator
+                cmark_iter_reset(iter, block, CMARK_EVENT_EXIT);
+
+            } else {
+                // no end found, but lets try sibling nodes
+                for (auto next = cmark_node_next(cur); next != nullptr;
+                     next      = cmark_node_next(next)) {
+                    // only text nodes again
+                    if (cmark_node_get_type(next) != CMARK_NODE_TEXT)
+                        continue;
+
+                    std::string_view next_content = cmark_node_get_literal(next);
+                    if (auto posEndNext = next_content.find("||");
+                        posEndNext != std::string_view::npos) {
+                        // We found the end of the spoiler
+                        std::string before_spoiler = std::string(content.substr(0, posStart));
+                        std::string after_spoiler =
+                          std::string(next_content.substr(posEndNext + 2));
+
+                        std::string inside_spoiler_start =
+                          std::string(content.substr(posStart + 2));
+                        std::string inside_spoiler_end =
+                          std::string(next_content.substr(0, posEndNext));
+
+                        std::string spoiler_text = extract_spoiler_warning(inside_spoiler_start);
+
+                        // save all the nodes inside the spoiler for later
+                        std::vector<cmark_node *> child_nodes;
+                        for (auto kid = cmark_node_next(cur); kid != nullptr && kid != next;
+                             kid      = cmark_node_next(kid)) {
+                            child_nodes.push_back(kid);
+                        }
+
+                        // create the new nodes
+                        auto before_node = cmark_node_new(cmark_node_type::CMARK_NODE_TEXT);
+                        cmark_node_set_literal(before_node, before_spoiler.c_str());
+                        auto after_node = cmark_node_new(cmark_node_type::CMARK_NODE_TEXT);
+                        cmark_node_set_literal(after_node, after_spoiler.c_str());
+
+                        auto block = cmark_node_new(cmark_node_type::CMARK_NODE_CUSTOM_INLINE);
+                        cmark_node_set_on_enter(
+                          block, ("<span data-mx-spoiler=\"" + spoiler_text + "\">").c_str());
+                        cmark_node_set_on_exit(block, "</span>");
+
+                        // create the content inside the spoiler by adding the old text at the start
+                        // and the end as well as all the existing children
+                        auto child_node_start = cmark_node_new(cmark_node_type::CMARK_NODE_TEXT);
+                        cmark_node_set_literal(child_node_start, inside_spoiler_start.c_str());
+                        cmark_node_append_child(block, child_node_start);
+                        for (auto &child : child_nodes)
+                            cmark_node_append_child(block, child);
+                        auto child_node_end = cmark_node_new(cmark_node_type::CMARK_NODE_TEXT);
+                        cmark_node_set_literal(child_node_end, inside_spoiler_end.c_str());
+                        cmark_node_append_child(block, child_node_end);
+
+                        // insert the new nodes into the tree
+                        cmark_node_replace(cur, block);
+                        cmark_node_insert_before(block, before_node);
+                        cmark_node_insert_after(block, after_node);
+
+                        // cleanup removed nodes
+                        cmark_node_free(cur);
+                        cmark_node_free(next);
+
+                        // fixup the iterator
+                        cmark_iter_reset(iter, block, CMARK_EVENT_EXIT);
+
+                        break;
+                    }
+                }
+            }
+        }
+    }
+
+    cmark_iter_free(iter);
+}
+
+static void
+process_strikethrough(cmark_node *node)
+{
+    auto iter = cmark_iter_new(node);
+
+    while (cmark_iter_next(iter) != CMARK_EVENT_DONE) {
+        cmark_node *cur = cmark_iter_get_node(iter);
+
+        // only text nodes (no code or similar)
+        if (cmark_node_get_type(cur) != CMARK_NODE_TEXT) {
+            continue;
+        }
+
+        std::string_view content = cmark_node_get_literal(cur);
+
+        if (auto posStart = content.find("~~"); posStart != std::string::npos) {
+            // we have the start of the strikethrough
+            if (auto posEnd = content.find("~~", posStart + 2); posEnd != std::string::npos) {
+                // we have the end of the strikethrough in the same node
+
+                std::string before_strike = std::string(content.substr(0, posStart));
+                std::string inside_strike =
+                  std::string(content.substr(posStart + 2, posEnd - 2 - posStart));
+                std::string after_strike = std::string(content.substr(posEnd + 2));
+
+                // create the new nodes
+                auto before_node = cmark_node_new(cmark_node_type::CMARK_NODE_TEXT);
+                cmark_node_set_literal(before_node, before_strike.c_str());
+                auto after_node = cmark_node_new(cmark_node_type::CMARK_NODE_TEXT);
+                cmark_node_set_literal(after_node, after_strike.c_str());
+
+                auto block = cmark_node_new(cmark_node_type::CMARK_NODE_CUSTOM_INLINE);
+                cmark_node_set_on_enter(block, "<del>");
+                cmark_node_set_on_exit(block, "</del>");
+                auto child_node = cmark_node_new(cmark_node_type::CMARK_NODE_TEXT);
+                cmark_node_set_literal(child_node, inside_strike.c_str());
+                cmark_node_append_child(block, child_node);
+
+                // insert the new nodes into the tree
+                cmark_node_replace(cur, block);
+                cmark_node_insert_before(block, before_node);
+                cmark_node_insert_after(block, after_node);
+
+                // cleanup the replaced node
+                cmark_node_free(cur);
+
+                // fixup the iterator
+                cmark_iter_reset(iter, block, CMARK_EVENT_EXIT);
+
+            } else {
+                // no end found, but lets try sibling nodes
+                for (auto next = cmark_node_next(cur); next != nullptr;
+                     next      = cmark_node_next(next)) {
+                    // only text nodes again
+                    if (cmark_node_get_type(next) != CMARK_NODE_TEXT)
+                        continue;
+
+                    std::string_view next_content = cmark_node_get_literal(next);
+                    if (auto posEndNext = next_content.find("~~");
+                        posEndNext != std::string_view::npos) {
+                        // We found the end of the strikethrough
+                        std::string before_strike = std::string(content.substr(0, posStart));
+                        std::string after_strike = std::string(next_content.substr(posEndNext + 2));
+
+                        std::string inside_strike_start = std::string(content.substr(posStart + 2));
+                        std::string inside_strike_end =
+                          std::string(next_content.substr(0, posEndNext));
+
+                        // save all the nodes inside the strikethrough for later
+                        std::vector<cmark_node *> child_nodes;
+                        for (auto kid = cmark_node_next(cur); kid != nullptr && kid != next;
+                             kid      = cmark_node_next(kid)) {
+                            child_nodes.push_back(kid);
+                        }
+
+                        // create the new nodes
+                        auto before_node = cmark_node_new(cmark_node_type::CMARK_NODE_TEXT);
+                        cmark_node_set_literal(before_node, before_strike.c_str());
+                        auto after_node = cmark_node_new(cmark_node_type::CMARK_NODE_TEXT);
+                        cmark_node_set_literal(after_node, after_strike.c_str());
+
+                        auto block = cmark_node_new(cmark_node_type::CMARK_NODE_CUSTOM_INLINE);
+                        cmark_node_set_on_enter(block, "<del>");
+                        cmark_node_set_on_exit(block, "</del>");
+
+                        // create the content inside the strikethrough by adding the old text at the
+                        // start and the end as well as all the existing children
+                        auto child_node_start = cmark_node_new(cmark_node_type::CMARK_NODE_TEXT);
+                        cmark_node_set_literal(child_node_start, inside_strike_start.c_str());
+                        cmark_node_append_child(block, child_node_start);
+                        for (auto &child : child_nodes)
+                            cmark_node_append_child(block, child);
+                        auto child_node_end = cmark_node_new(cmark_node_type::CMARK_NODE_TEXT);
+                        cmark_node_set_literal(child_node_end, inside_strike_end.c_str());
+                        cmark_node_append_child(block, child_node_end);
+
+                        // insert the new nodes into the tree
+                        cmark_node_replace(cur, block);
+                        cmark_node_insert_before(block, before_node);
+                        cmark_node_insert_after(block, after_node);
+
+                        // cleanup removed nodes
+                        cmark_node_free(cur);
+                        cmark_node_free(next);
+
+                        // fixup the iterator
+                        cmark_iter_reset(iter, block, CMARK_EVENT_EXIT);
+
+                        break;
+                    }
+                }
+            }
+        }
+    }
+
+    cmark_iter_free(iter);
+}
 QString
-utils::markdownToHtml(const QString &text, bool rainbowify)
+utils::markdownToHtml(const QString &text, bool rainbowify_, bool noExtensions)
 {
     const auto str         = text.toUtf8();
     cmark_node *const node = cmark_parse_document(str.constData(), str.size(), CMARK_OPT_UNSAFE);
 
-    if (rainbowify) {
-        // create iterator over node
-        cmark_iter *iter = cmark_iter_new(node);
+    if (!noExtensions) {
+        process_strikethrough(node);
+        process_spoilers(node);
 
-        // First loop to get total text length
-        int textLen = 0;
-        while (cmark_iter_next(iter) != CMARK_EVENT_DONE) {
-            cmark_node *cur = cmark_iter_get_node(iter);
-            // only text nodes (no code or semilar)
-            if (cmark_node_get_type(cur) != CMARK_NODE_TEXT)
-                continue;
-            // count up by length of current node's text
-            QTextBoundaryFinder tbf(QTextBoundaryFinder::BoundaryType::Grapheme,
-                                    QString(cmark_node_get_literal(cur)));
-            while (tbf.toNextBoundary() != -1)
-                textLen++;
+        if (rainbowify_) {
+            rainbowify(node);
         }
-
-        // create new iter to start over
-        cmark_iter_free(iter);
-        iter = cmark_iter_new(node);
-
-        // Second loop to rainbowify
-        int charIdx = 0;
-        while (cmark_iter_next(iter) != CMARK_EVENT_DONE) {
-            cmark_node *cur = cmark_iter_get_node(iter);
-            // only text nodes (no code or semilar)
-            if (cmark_node_get_type(cur) != CMARK_NODE_TEXT)
-                continue;
-
-            // get text in current node
-            QString nodeText(cmark_node_get_literal(cur));
-            // create buffer to append rainbow text to
-            QString buf;
-            int boundaryStart = 0;
-            int boundaryEnd   = 0;
-            // use QTextBoundaryFinder to iterate ofer graphemes
-            QTextBoundaryFinder tbf(QTextBoundaryFinder::BoundaryType::Grapheme, nodeText);
-            while ((boundaryEnd = tbf.toNextBoundary()) != -1) {
-                charIdx++;
-                // Split text to get current char
-                auto curChar =
-                  QStringView(nodeText).mid(boundaryStart, boundaryEnd - boundaryStart);
-                boundaryStart = boundaryEnd;
-                // Don't rainbowify whitespaces
-                if (curChar.trimmed().isEmpty() || codepointIsEmoji(curChar.toUcs4().at(0))) {
-                    buf.append(curChar);
-                    continue;
-                }
-
-                // get correct color for char index
-                // Use colors as described here:
-                // https://shark.comfsm.fm/~dleeling/cis/hsl_rainbow.html
-                auto color = QColor::fromHslF((charIdx - 1.0) / textLen * (5. / 6.), 0.9, 0.5);
-                // format color for HTML
-                auto colorString = color.name(QColor::NameFormat::HexRgb);
-                // create HTML element for current char
-                auto curCharColored =
-                  QStringLiteral("<font color=\"%0\">%1</font>").arg(colorString).arg(curChar);
-                // append colored HTML element to buffer
-                buf.append(curCharColored);
-            }
-
-            // create HTML_INLINE node to prevent HTML from being escaped
-            auto htmlNode = cmark_node_new(CMARK_NODE_HTML_INLINE);
-            // set content of HTML node to buffer contents
-            cmark_node_set_literal(htmlNode, buf.toUtf8().data());
-            // replace current node with HTML node
-            cmark_node_replace(cur, htmlNode);
-            // free memory of old node
-            cmark_node_free(cur);
-        }
-
-        cmark_iter_free(iter);
     }
 
-    const char *tmp_buf = cmark_render_html(node, CMARK_OPT_UNSAFE);
+    const char *tmp_buf = cmark_render_html(
+      node,
+      // by default make single linebreaks <br> tags
+      noExtensions ? CMARK_OPT_UNSAFE : (CMARK_OPT_UNSAFE | CMARK_OPT_HARDBREAKS));
     // Copy the null terminated output buffer.
     std::string html(tmp_buf);
 
     // The buffer is no longer needed.
     free((char *)tmp_buf);
+    cmark_node_free(node);
 
-    auto result = linkifyMessage(escapeBlacklistedHtml(QString::fromStdString(html))).trimmed();
+    auto result = escapeBlacklistedHtml(QString::fromStdString(html)).trimmed();
+
+    if (!noExtensions) {
+        result = linkifyMessage(std::move(result)).trimmed();
+    }
 
     if (result.count(QStringLiteral("<p>")) == 1 && result.startsWith(QLatin1String("<p>")) &&
         result.endsWith(QLatin1String("</p>"))) {
@@ -763,15 +1032,15 @@ utils::generateContrastingHexColor(const QString &input, const QColor &backgroun
     auto hash = hashQString(input);
     // create a hue value based on the hash of the input.
     // Adapted to make Nico blue
-    auto userHue =
-      static_cast<int>(static_cast<double>(hash - static_cast<uint32_t>(0x60'00'00'00)) /
-                       std::numeric_limits<uint32_t>::max() * 360.);
+    auto userHue = static_cast<double>(hash - static_cast<uint32_t>(0x60'00'00'00)) /
+                   std::numeric_limits<uint32_t>::max() * 360.;
     // start with moderate saturation and lightness values.
-    auto sat       = 230;
-    auto lightness = 125;
+    auto sat       = 230.;
+    auto lightness = 125.;
 
     // converting to a QColor makes the luminance calc easier.
-    QColor inputColor = QColor::fromHsl(userHue, sat, lightness);
+    QColor inputColor = QColor::fromHsl(
+      static_cast<int>(userHue), static_cast<int>(sat), static_cast<int>(lightness));
 
     // calculate the initial luminance and contrast of the
     // generated color.  It's possible that no additional
@@ -789,7 +1058,9 @@ utils::generateContrastingHexColor(const QString &input, const QColor &backgroun
         if (lightness >= 242 || lightness <= 13) {
             qreal newSat = qBound(26.0, sat * 1.25, 242.0);
 
-            inputColor.setHsl(userHue, qFloor(newSat), lightness);
+            inputColor.setHsl(static_cast<int>(userHue),
+                              static_cast<int>(qFloor(newSat)),
+                              static_cast<int>(lightness));
             auto tmpLum         = luminance(inputColor);
             auto higherContrast = computeContrast(tmpLum, backgroundLum);
             if (higherContrast > contrast) {
@@ -797,7 +1068,9 @@ utils::generateContrastingHexColor(const QString &input, const QColor &backgroun
                 sat      = newSat;
             } else {
                 newSat = qBound(26.0, sat / 1.25, 242.0);
-                inputColor.setHsl(userHue, qFloor(newSat), lightness);
+                inputColor.setHsl(static_cast<int>(userHue),
+                                  static_cast<int>(qFloor(newSat)),
+                                  static_cast<int>(lightness));
                 tmpLum             = luminance(inputColor);
                 auto lowerContrast = computeContrast(tmpLum, backgroundLum);
                 if (lowerContrast > contrast) {
@@ -808,7 +1081,9 @@ utils::generateContrastingHexColor(const QString &input, const QColor &backgroun
         } else {
             qreal newLightness = qBound(13.0, lightness * 1.25, 242.0);
 
-            inputColor.setHsl(userHue, sat, qFloor(newLightness));
+            inputColor.setHsl(static_cast<int>(userHue),
+                              static_cast<int>(sat),
+                              static_cast<int>(qFloor(newLightness)));
 
             auto tmpLum         = luminance(inputColor);
             auto higherContrast = computeContrast(tmpLum, backgroundLum);
@@ -820,7 +1095,9 @@ utils::generateContrastingHexColor(const QString &input, const QColor &backgroun
                 // otherwise, try going the other way instead.
             } else {
                 newLightness = qBound(13.0, lightness / 1.25, 242.0);
-                inputColor.setHsl(userHue, sat, qFloor(newLightness));
+                inputColor.setHsl(static_cast<int>(userHue),
+                                  static_cast<int>(sat),
+                                  static_cast<int>(qFloor(newLightness)));
                 tmpLum             = luminance(inputColor);
                 auto lowerContrast = computeContrast(tmpLum, backgroundLum);
                 if (lowerContrast > contrast) {
@@ -879,8 +1156,8 @@ utils::centerWidget(QWidget *widget, QWindow *parent)
     }
 
     auto findCenter = [childRect = widget->rect()](QRect hostRect) -> QPoint {
-        return QPoint(hostRect.center().x() - (childRect.width() * 0.5),
-                      hostRect.center().y() - (childRect.height() * 0.5));
+        return QPoint(static_cast<int>(hostRect.center().x() - (childRect.width() * 0.5)),
+                      static_cast<int>(hostRect.center().y() - (childRect.height() * 0.5)));
     };
     widget->move(findCenter(QGuiApplication::primaryScreen()->geometry()));
 }
