@@ -1,4 +1,5 @@
 // SPDX-FileCopyrightText: 2022 Nheko Contributors
+// SPDX-FileCopyrightText: 2023 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -13,7 +14,7 @@
 
 class RoomlistModel;
 
-class NhekoDBusBackend : public QObject
+class NhekoDBusBackend final : public QObject
 {
     Q_OBJECT
     Q_CLASSINFO("D-Bus Interface", "im.nheko.Nheko")
@@ -27,7 +28,9 @@ public slots:
     //! Get the nheko version.
     Q_SCRIPTABLE QString nhekoVersion() const { return nheko::version; }
     //! Call this function to get a list of all joined rooms.
-    Q_SCRIPTABLE QVector<nheko::dbus::RoomInfoItem> rooms(const QDBusMessage &message);
+    Q_SCRIPTABLE QVector<nheko::dbus::RoomInfoItem> rooms() const;
+    //! Call this function to convert a URI into an image
+    Q_SCRIPTABLE QImage image(const QString &uri, const QDBusMessage &message) const;
     //! Activates a currently joined room.
     Q_SCRIPTABLE void activateRoom(const QString &alias) const;
     //! Joins a room. It is your responsibility to ask for confirmation (if desired).
@@ -35,6 +38,8 @@ public slots:
     //! Starts or activates a direct chat. It is your responsibility to ask for confirmation (if
     //! desired).
     Q_SCRIPTABLE void directChat(const QString &userId) const;
+    //! Sets the user's status message.
+    Q_SCRIPTABLE void setStatusMessage(const QString &message);
 
 private:
     void bringWindowToTop() const;
