@@ -1,5 +1,6 @@
 // SPDX-FileCopyrightText: 2021 Nheko Contributors
 // SPDX-FileCopyrightText: 2022 Nheko Contributors
+// SPDX-FileCopyrightText: 2023 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -66,10 +67,10 @@ ReCaptcha::ReCaptcha(const QString &session, QWidget *parent)
 
     connect(confirmBtn_, &QPushButton::clicked, this, [this]() {
         emit confirmation();
-        emit close();
+        close();
     });
     connect(cancelBtn_, &QPushButton::clicked, this, [this]() {
         emit cancel();
-        emit close();
+        close();
     });
 }
