@@ -1,5 +1,6 @@
 // SPDX-FileCopyrightText: 2021 Nheko Contributors
 // SPDX-FileCopyrightText: 2022 Nheko Contributors
+// SPDX-FileCopyrightText: 2023 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -11,7 +12,7 @@ class QPushButton;
 
 namespace dialogs {
 
-class ReCaptcha : public QWidget
+class ReCaptcha final : public QWidget
 {
     Q_OBJECT
 
