@@ -1,4 +1,5 @@
 // SPDX-FileCopyrightText: 2022 Nheko Contributors
+// SPDX-FileCopyrightText: 2023 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -10,7 +11,7 @@
 #include <QtDBus/QDBusInterface>
 #endif
 
-class Dock : public QObject
+class Dock final : public QObject
 {
     Q_OBJECT
 public:
