@@ -1,11 +1,13 @@
 // SPDX-FileCopyrightText: 2021 Nheko Contributors
 // SPDX-FileCopyrightText: 2022 Nheko Contributors
+// SPDX-FileCopyrightText: 2023 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
 #include "Olm.h"
 
 #include <QObject>
+#include <QRandomGenerator>
 #include <QTimer>
 
 #include <nlohmann/json.hpp>
@@ -30,6 +32,7 @@ auto client_ = std::make_unique<mtx::crypto::OlmClient>();
 std::map<std::string, std::string> request_id_to_secret_name;
 
 constexpr auto MEGOLM_ALGO = "m.megolm.v1.aes-sha2";
+constexpr auto OLM_ALGO    = "m.olm.v1.curve25519-aes-sha2";
 }
 
 namespace olm {
@@ -98,13 +101,19 @@ handle_secret_request(const mtx::events::DeviceEvent<mtx::events::msg::SecretReq
         return;
     secretSend.content.secret = secret.value();
 
-    send_encrypted_to_device_messages(
-      {{local_user.to_string(), {{e->content.requesting_device_id}}}}, secretSend);
+    // Randomly delay reply to workaround olm session generation races
+    QTimer::singleShot(QRandomGenerator::global()->bounded(0, 3000),
+                       ChatPage::instance(),
+                       [local_user, e = *e, secretSend] {
+                           send_encrypted_to_device_messages(
+                             {{local_user.to_string(), {{e.content.requesting_device_id}}}},
+                             secretSend);
 
-    nhlog::net()->info("Sent secret '{}' to ({},{})",
-                       e->content.name,
-                       local_user.to_string(),
-                       e->content.requesting_device_id);
+                           nhlog::net()->info("Sent secret '{}' to ({},{})",
+                                              e.content.name,
+                                              local_user.to_string(),
+                                              e.content.requesting_device_id);
+                       });
 }
 
 void
@@ -130,8 +139,8 @@ handle_to_device_messages(const std::vector<mtx::events::collections::DeviceEven
                 cache::client()->query_keys(
                   olm_msg.sender, [olm_msg](const UserKeyCache &userKeys, mtx::http::RequestErr e) {
                       if (e) {
-                          nhlog::crypto()->error("Failed to query user keys, dropping olm "
-                                                 "message");
+                          nhlog::crypto()->error(
+                            "Failed to query user keys, dropping olm message: {}", e);
                           return;
                       }
                       handle_olm_message(std::move(olm_msg), userKeys);
@@ -888,12 +897,15 @@ download_full_keybackup()
 {
     if (!UserSettings::instance()->useOnlineKeyBackup()) {
         // Online key backup disabled
+        nhlog::crypto()->debug("Not downloading full online key backup, because it is disabled.");
         return;
     }
 
     auto backupVersion = cache::client()->backupVersion();
     if (!backupVersion) {
         // no trusted OKB
+        nhlog::crypto()->debug(
+          "Not downloading full online key backup, because we don't have a version for it.");
         return;
     }
 
@@ -902,10 +914,14 @@ download_full_keybackup()
     auto decryptedSecret = cache::secret(mtx::secret_storage::secrets::megolm_backup_v1);
     if (!decryptedSecret) {
         // no backup key available
+        nhlog::crypto()->debug(
+          "Not downloading full online key backup, because we don't have a key for it.");
         return;
     }
     auto sessionDecryptionKey = to_binary_buf(base642bin(*decryptedSecret));
 
+    nhlog::crypto()->debug("Downloading full online key backup.");
+
     http::client()->room_keys(
       backupVersion->version,
       [sessionDecryptionKey](const mtx::responses::backup::KeysBackup &bk,
@@ -917,11 +933,12 @@ download_full_keybackup()
                                          err->matrix_error.error);
               return;
           }
+          nhlog::crypto()->debug("Storing full online key backup.");
 
           mtx::crypto::ExportedSessionKeys allKeys;
-          try {
-              for (const auto &[room, roomKey] : bk.rooms) {
-                  for (const auto &[session_id, encSession] : roomKey.sessions) {
+          for (const auto &[room, roomKey] : bk.rooms) {
+              for (const auto &[session_id, encSession] : roomKey.sessions) {
+                  try {
                       auto session = decrypt_session(encSession.session_data, sessionDecryptionKey);
 
                       if (session.algorithm != mtx::crypto::MEGOLM_ALGO)
@@ -938,20 +955,26 @@ download_full_keybackup()
                       sess.sender_key          = std::move(session.sender_key);
                       sess.session_key         = std::move(session.session_key);
                       allKeys.sessions.push_back(std::move(sess));
+                  } catch (const olm_exception &e) {
+                      nhlog::crypto()->critical("failed to decrypt inbound megolm session: {}",
+                                                e.what());
                   }
               }
-
-              // call on UI thread
-              QTimer::singleShot(0, ChatPage::instance(), [keys = std::move(allKeys)] {
-                  cache::importSessionKeys(keys);
-              });
-          } catch (const lmdb::error &e) {
-              nhlog::crypto()->critical("failed to save inbound megolm session: {}", e.what());
           }
+
+          // call on UI thread
+          QTimer::singleShot(0, ChatPage::instance(), [keys = std::move(allKeys)] {
+              try {
+                  cache::importSessionKeys(keys);
+                  nhlog::crypto()->debug("Storing full online key backup completed.");
+              } catch (const lmdb::error &e) {
+                  nhlog::crypto()->critical("failed to save inbound megolm session: {}", e.what());
+              }
+          });
       });
 }
 void
-lookup_keybackup(const std::string room, const std::string session_id)
+lookup_keybackup(const std::string &room, const std::string &session_id)
 {
     if (!UserSettings::instance()->useOnlineKeyBackup()) {
         // Online key backup disabled
@@ -1075,8 +1098,6 @@ send_key_request_for(mtx::events::EncryptedEvent<mtx::events::msg::Encrypted> e,
           nhlog::net()->info(
             "m.room_key_request sent to {}:{} and your own devices", e.sender, e.content.device_id);
       });
-
-    // http::client()->room_keys
 }
 
 void
@@ -1144,7 +1165,7 @@ handle_key_request_message(const mtx::events::DeviceEvent<mtx::events::msg::KeyR
     }
 
     bool shouldSeeKeys    = false;
-    uint64_t minimumIndex = -1;
+    uint32_t minimumIndex = -1;
     if (sessionData->currently.keys.count(req.sender)) {
         if (sessionData->currently.keys.at(req.sender)
               .deviceids.count(req.content.requesting_device_id)) {
@@ -1283,7 +1304,7 @@ calculate_trust(const std::string &user_id, const MegolmSessionIndex &index)
 //! Send encrypted to device messages, targets is a map from userid to device ids or {} for all
 //! devices
 void
-send_encrypted_to_device_messages(const std::map<std::string, std::vector<std::string>> targets,
+send_encrypted_to_device_messages(const std::map<std::string, std::vector<std::string>> &targets,
                                   const mtx::events::collections::DeviceEvents &event,
                                   bool force_new_session)
 {
@@ -1299,78 +1320,83 @@ send_encrypted_to_device_messages(const std::map<std::string, std::vector<std::s
 
     auto our_curve = olm::client()->identity_keys().curve25519;
 
-    for (const auto &[user, devices] : targets) {
-        auto deviceKeys = cache::client()->userKeys(user);
+    {
+        auto currentTime = QDateTime::currentSecsSinceEpoch();
+        std::vector<std::pair<std::string, mtx::crypto::OlmSessionPtr>> sessionsToPersist;
 
-        // no keys for user, query them
-        if (!deviceKeys) {
-            keysToQuery[user] = devices;
-            continue;
-        }
+        for (const auto &[user, devices] : targets) {
+            auto deviceKeys = cache::client()->userKeys(user);
 
-        auto deviceTargets = devices;
-        if (devices.empty()) {
-            deviceTargets.clear();
-            deviceTargets.reserve(deviceKeys->device_keys.size());
-            for (const auto &[device, keys] : deviceKeys->device_keys) {
-                (void)keys;
-                deviceTargets.push_back(device);
-            }
-        }
-
-        for (const auto &device : deviceTargets) {
-            if (!deviceKeys->device_keys.count(device)) {
-                keysToQuery[user] = {};
-                break;
-            }
-
-            auto d = deviceKeys->device_keys.at(device);
-
-            if (!d.keys.count("curve25519:" + device) || !d.keys.count("ed25519:" + device)) {
-                nhlog::crypto()->warn("Skipping device {} since it has no keys!", device);
+            // no keys for user, query them
+            if (!deviceKeys) {
+                keysToQuery[user] = devices;
                 continue;
             }
 
-            auto device_curve = d.keys.at("curve25519:" + device);
-            if (device_curve == our_curve) {
-                nhlog::crypto()->warn("Skipping our own device, since sending "
-                                      "ourselves olm messages makes no sense.");
-                continue;
-            }
-
-            auto session = cache::getLatestOlmSession(device_curve);
-            if (!session || force_new_session) {
-                auto currentTime = QDateTime::currentSecsSinceEpoch();
-                if (rateLimit.value(QPair(user, device)) + 60 * 60 * 10 < currentTime) {
-                    claims.one_time_keys[user][device] = mtx::crypto::SIGNED_CURVE25519;
-                    pks[user][device].ed25519          = d.keys.at("ed25519:" + device);
-                    pks[user][device].curve25519       = d.keys.at("curve25519:" + device);
-
-                    rateLimit.insert(QPair(user, device), currentTime);
-                } else {
-                    nhlog::crypto()->warn("Not creating new session with {}:{} "
-                                          "because of rate limit",
-                                          user,
-                                          device);
+            auto deviceTargets = devices;
+            if (devices.empty()) {
+                deviceTargets.clear();
+                deviceTargets.reserve(deviceKeys->device_keys.size());
+                for (const auto &[device, keys] : deviceKeys->device_keys) {
+                    (void)keys;
+                    deviceTargets.push_back(device);
                 }
-                continue;
             }
 
-            messages[mtx::identifiers::parse<mtx::identifiers::User>(user)][device] =
-              olm::client()
-                ->create_olm_encrypted_content(session->get(),
-                                               ev_json,
-                                               UserId(user),
-                                               d.keys.at("ed25519:" + device),
-                                               device_curve)
-                .get<mtx::events::msg::OlmEncrypted>();
+            for (const auto &device : deviceTargets) {
+                if (!deviceKeys->device_keys.count(device)) {
+                    keysToQuery[user] = {};
+                    break;
+                }
 
+                const auto &d = deviceKeys->device_keys.at(device);
+
+                if (!d.keys.count("curve25519:" + device) || !d.keys.count("ed25519:" + device)) {
+                    nhlog::crypto()->warn("Skipping device {} since it has no keys!", device);
+                    continue;
+                }
+
+                auto device_curve = d.keys.at("curve25519:" + device);
+                if (device_curve == our_curve) {
+                    nhlog::crypto()->warn("Skipping our own device, since sending "
+                                          "ourselves olm messages makes no sense.");
+                    continue;
+                }
+
+                auto session = cache::getLatestOlmSession(device_curve);
+                if (!session || force_new_session) {
+                    if (rateLimit.value(QPair(user, device)) + 60 * 60 * 10 < currentTime) {
+                        claims.one_time_keys[user][device] = mtx::crypto::SIGNED_CURVE25519;
+                        pks[user][device].ed25519          = d.keys.at("ed25519:" + device);
+                        pks[user][device].curve25519       = d.keys.at("curve25519:" + device);
+
+                        rateLimit.insert(QPair(user, device), currentTime);
+                    } else {
+                        nhlog::crypto()->warn("Not creating new session with {}:{} "
+                                              "because of rate limit",
+                                              user,
+                                              device);
+                    }
+                    continue;
+                }
+
+                messages[mtx::identifiers::parse<mtx::identifiers::User>(user)][device] =
+                  olm::client()
+                    ->create_olm_encrypted_content(session->get(),
+                                                   ev_json,
+                                                   UserId(user),
+                                                   d.keys.at("ed25519:" + device),
+                                                   device_curve)
+                    .get<mtx::events::msg::OlmEncrypted>();
+                sessionsToPersist.emplace_back(d.keys.at("curve25519:" + device),
+                                               std::move(*session));
+            }
+        }
+
+        if (!sessionsToPersist.empty()) {
             try {
-                nhlog::crypto()->debug("Updated olm session: {}",
-                                       mtx::crypto::session_id(session->get()));
-                cache::saveOlmSession(d.keys.at("curve25519:" + device),
-                                      std::move(*session),
-                                      QDateTime::currentMSecsSinceEpoch());
+                nhlog::crypto()->debug("Updated olm sessions: {}", sessionsToPersist.size());
+                cache::client()->saveOlmSessions(std::move(sessionsToPersist), currentTime);
             } catch (const lmdb::error &e) {
                 nhlog::db()->critical("failed to save outbound olm session: {}", e.what());
             } catch (const mtx::crypto::olm_exception &e) {
@@ -1395,6 +1421,9 @@ send_encrypted_to_device_messages(const std::map<std::string, std::vector<std::s
                                          mtx::http::RequestErr) {
             std::map<mtx::identifiers::User, std::map<std::string, mtx::events::msg::OlmEncrypted>>
               messages;
+            auto currentTime = QDateTime::currentSecsSinceEpoch();
+            std::vector<std::pair<std::string, mtx::crypto::OlmSessionPtr>> sessionsToPersist;
+
             for (const auto &[user_id, retrieved_devices] : res.one_time_keys) {
                 nhlog::net()->debug("claimed keys for {}", user_id);
                 if (retrieved_devices.size() == 0) {
@@ -1440,21 +1469,24 @@ send_encrypted_to_device_messages(const std::map<std::string, std::vector<std::s
                           session.get(), ev_json, UserId(user_id), sign_key, id_key)
                         .get<mtx::events::msg::OlmEncrypted>();
 
-                    try {
-                        nhlog::crypto()->debug("Updated olm session: {}",
-                                               mtx::crypto::session_id(session.get()));
-                        cache::saveOlmSession(
-                          id_key, std::move(session), QDateTime::currentMSecsSinceEpoch());
-                    } catch (const lmdb::error &e) {
-                        nhlog::db()->critical("failed to save outbound olm session: {}", e.what());
-                    } catch (const mtx::crypto::olm_exception &e) {
-                        nhlog::crypto()->critical("failed to pickle outbound olm session: {}",
-                                                  e.what());
-                    }
+                    sessionsToPersist.emplace_back(id_key, std::move(session));
                 }
                 nhlog::net()->info("send_to_device: {}", user_id);
             }
 
+            if (!sessionsToPersist.empty()) {
+                try {
+                    nhlog::crypto()->debug("Updated (new) olm sessions: {}",
+                                           sessionsToPersist.size());
+                    cache::client()->saveOlmSessions(std::move(sessionsToPersist), currentTime);
+                } catch (const lmdb::error &e) {
+                    nhlog::db()->critical("failed to save outbound olm session: {}", e.what());
+                } catch (const mtx::crypto::olm_exception &e) {
+                    nhlog::crypto()->critical("failed to pickle outbound olm session: {}",
+                                              e.what());
+                }
+            }
+
             if (!messages.empty())
                 http::client()->send_to_device<mtx::events::msg::OlmEncrypted>(
                   http::client()->generate_txn_id(), messages, [](mtx::http::RequestErr err) {
@@ -1594,6 +1626,11 @@ request_cross_signing_keys()
                 body[local_user][dev] = secretRequest;
         }
 
+        if (body.empty()) {
+            nhlog::net()->warn("No verified devices to request {} from.", secretName);
+            return;
+        }
+
         http::client()->send_to_device<mtx::events::msg::SecretRequest>(
           http::client()->generate_txn_id(),
           body,
