@@ -1,5 +1,6 @@
 // SPDX-FileCopyrightText: 2021 Nheko Contributors
 // SPDX-FileCopyrightText: 2022 Nheko Contributors
+// SPDX-FileCopyrightText: 2023 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -7,7 +8,7 @@
 
 #include <QObject>
 
-class SelfVerificationStatus : public QObject
+class SelfVerificationStatus final : public QObject
 {
     Q_OBJECT
 
@@ -25,7 +26,8 @@ public:
     };
     Q_ENUM(Status)
 
-    Q_INVOKABLE void setupCrosssigning(bool useSSSS, QString password, bool useOnlineKeyBackup);
+    Q_INVOKABLE void
+    setupCrosssigning(bool useSSSS, const QString &password, bool useOnlineKeyBackup);
     Q_INVOKABLE void verifyMasterKey();
     Q_INVOKABLE void verifyMasterKeyWithPassphrase();
     Q_INVOKABLE void verifyUnverifiedDevices();
