@@ -1,5 +1,6 @@
 // SPDX-FileCopyrightText: 2021 Nheko Contributors
 // SPDX-FileCopyrightText: 2022 Nheko Contributors
+// SPDX-FileCopyrightText: 2023 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -18,7 +19,7 @@ class TimelineModel;
 class TimelineViewManager;
 class RoomlistModel;
 
-class VerificationManager : public QObject
+class VerificationManager final : public QObject
 {
     Q_OBJECT
 
