@@ -1,5 +1,6 @@
 // SPDX-FileCopyrightText: 2021 Nheko Contributors
 // SPDX-FileCopyrightText: 2022 Nheko Contributors
+// SPDX-FileCopyrightText: 2023 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -7,6 +8,7 @@
 
 #include "Cache.h"
 #include "EventAccessors.h"
+#include "Logging.h"
 #include "Utils.h"
 
 QString
@@ -33,3 +35,24 @@ NotificationsManager::getMessageTemplate(const mtx::responses::Notification &not
         return QStringLiteral("%1: %2").arg(sender);
     }
 }
+
+void
+NotificationsManager::removeNotifications(const QString &roomId_,
+                                          const std::vector<QString> &eventIds)
+{
+    std::string room_id = roomId_.toStdString();
+
+    std::uint64_t markerPos = 0;
+    for (const auto &e : eventIds) {
+        markerPos = std::max(markerPos, cache::getEventIndex(room_id, e.toStdString()).value_or(0));
+    }
+
+    for (const auto &[roomId, eventId] : qAsConst(this->notificationIds)) {
+        if (roomId != roomId_)
+            continue;
+        auto idx = cache::getEventIndex(room_id, eventId.toStdString());
+        if (!idx || markerPos >= idx) {
+            removeNotification(roomId, eventId);
+        }
+    }
+}
