@@ -271,3 +271,13 @@ NotificationsManager::notificationClosed(uint id, uint reason)
     Q_UNUSED(reason);
     notificationIds.remove(id);
 }
+
+void
+NotificationsManager::closeAllNotifications()
+{
+    const auto ids = notificationIds.keys();
+    for (const auto &id : ids) {
+        closeNotification(id);
+        notificationIds.remove(id);
+    }
+}
