@@ -1,5 +1,6 @@
 // SPDX-FileCopyrightText: 2021 Nheko Contributors
 // SPDX-FileCopyrightText: 2022 Nheko Contributors
+// SPDX-FileCopyrightText: 2023 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -153,7 +154,8 @@ CommunitiesModel::data(const QModelIndex &index, int role) const
             int count = 0;
             auto end  = spaceOrder_.lastChild(index.row() - 2);
             for (int i = index.row() - 2; i <= end; i++)
-                count += spaceOrder_.tree[i].notificationCounts.notification_count;
+                count +=
+                  static_cast<int>(spaceOrder_.tree[i].notificationCounts.notification_count);
             return count;
         }
         case CommunitiesModel::Roles::HasLoudNotification: {
@@ -577,6 +579,9 @@ CommunitiesModel::sync(const mtx::responses::Sync &sync_)
 void
 CommunitiesModel::setCurrentTagId(const QString &tagId)
 {
+    if (currentTagId_ == tagId)
+        return;
+
     if (tagId.startsWith(QLatin1String("tag:"))) {
         auto tag = tagId.mid(4);
         for (const auto &t : qAsConst(tags_)) {
@@ -605,6 +610,20 @@ CommunitiesModel::setCurrentTagId(const QString &tagId)
     emit currentTagIdChanged(currentTagId_);
 }
 
+bool
+CommunitiesModel::trySwitchToSpace(const QString &tag)
+{
+    for (const auto &t : spaceOrder_.tree) {
+        if (t.id == tag) {
+            this->currentTagId_ = "space:" + tag;
+            emit currentTagIdChanged(currentTagId_);
+            return true;
+        }
+    }
+
+    return false;
+}
+
 void
 CommunitiesModel::toggleTagId(QString tagId)
 {
@@ -679,7 +698,7 @@ enum Categories
 };
 
 Categories
-tagIdToCat(QString tagId)
+tagIdToCat(const QString &tagId)
 {
     if (tagId.isEmpty())
         return World;
@@ -743,7 +762,7 @@ FilteredCommunitiesModel::filterAcceptsRow(int sourceRow, const QModelIndex &) c
 }
 
 QVariantList
-CommunitiesModel::spaceChildrenListFromIndex(QString room, int idx) const
+CommunitiesModel::spaceChildrenListFromIndex(const QString &room, int idx) const
 {
     if (idx < -1)
         return {};
@@ -794,7 +813,7 @@ CommunitiesModel::spaceChildrenListFromIndex(QString room, int idx) const
         }
     }
 
-    nhlog::ui()->critical("Returning {} spaces", ret.size());
+    // nhlog::ui()->critical("Returning {} spaces", ret.size());
     return ret;
 }
 
@@ -834,11 +853,12 @@ CommunitiesModel::updateSpaceStatus(QString space,
               [space, room](mtx::responses::EventId, mtx::http::RequestErr err) {
                   if (err) {
                       ChatPage::instance()->showNotification(
-                        tr("Failed to update space child: %1")
+                        tr("Failed to update community: %1")
                           .arg(QString::fromStdString(err->matrix_error.error)));
                       nhlog::net()->error("Failed to update child {} of {}: {}",
                                           room.toStdString(),
-                                          space.toStdString());
+                                          space.toStdString(),
+                                          *err);
                   }
               });
         }
@@ -851,11 +871,12 @@ CommunitiesModel::updateSpaceStatus(QString space,
               [space, room](mtx::responses::EventId, mtx::http::RequestErr err) {
                   if (err) {
                       ChatPage::instance()->showNotification(
-                        tr("Failed to delete space child: %1")
+                        tr("Failed to delete room from community: %1")
                           .arg(QString::fromStdString(err->matrix_error.error)));
                       nhlog::net()->error("Failed to delete child {} of {}: {}",
                                           room.toStdString(),
-                                          space.toStdString());
+                                          space.toStdString(),
+                                          *err);
                   }
               });
         }
@@ -873,11 +894,12 @@ CommunitiesModel::updateSpaceStatus(QString space,
               [space, room](mtx::responses::EventId, mtx::http::RequestErr err) {
                   if (err) {
                       ChatPage::instance()->showNotification(
-                        tr("Failed to update space parent: %1")
+                        tr("Failed to update community for room: %1")
                           .arg(QString::fromStdString(err->matrix_error.error)));
                       nhlog::net()->error("Failed to update parent {} of {}: {}",
                                           space.toStdString(),
-                                          room.toStdString());
+                                          room.toStdString(),
+                                          *err);
                   }
               });
         }
@@ -890,11 +912,12 @@ CommunitiesModel::updateSpaceStatus(QString space,
               [space, room](mtx::responses::EventId, mtx::http::RequestErr err) {
                   if (err) {
                       ChatPage::instance()->showNotification(
-                        tr("Failed to delete space parent: %1")
+                        tr("Failed to remove community from room: %1")
                           .arg(QString::fromStdString(err->matrix_error.error)));
                       nhlog::net()->error("Failed to delete parent {} of {}: {}",
                                           space.toStdString(),
-                                          room.toStdString());
+                                          room.toStdString(),
+                                          *err);
                   }
               });
         }
