@@ -1,5 +1,6 @@
 // SPDX-FileCopyrightText: 2021 Nheko Contributors
 // SPDX-FileCopyrightText: 2022 Nheko Contributors
+// SPDX-FileCopyrightText: 2023 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -25,21 +26,32 @@ class CombinedImagePackModel;
 class QMimeData;
 class QDropEvent;
 
+struct DeleteLaterDeleter
+{
+    void operator()(QObject *p)
+    {
+        if (p)
+            p->deleteLater();
+    }
+};
+
 enum class MarkdownOverride
 {
     NOT_SPECIFIED, // no override set
     ON,
     OFF,
+    CMARK,
 };
 
-class InputVideoSurface : public QAbstractVideoSurface
+class InputVideoSurface final : public QAbstractVideoSurface
 {
     Q_OBJECT
 
 public:
     InputVideoSurface(QObject *parent)
       : QAbstractVideoSurface(parent)
-    {}
+    {
+    }
 
     bool present(const QVideoFrame &frame) override;
 
@@ -50,7 +62,7 @@ signals:
     void newImage(QImage img);
 };
 
-class MediaUpload : public QObject
+class MediaUpload final : public QObject
 {
     Q_OBJECT
     Q_PROPERTY(int mediaType READ type NOTIFY mediaTypeChanged)
@@ -73,8 +85,8 @@ public:
     Q_ENUM(MediaType)
 
     explicit MediaUpload(std::unique_ptr<QIODevice> data,
-                         QString mimetype,
-                         QString originalFilename,
+                         const QString &mimetype,
+                         const QString &originalFilename,
                          bool encrypt,
                          QObject *parent = nullptr);
 
@@ -158,7 +170,7 @@ public:
     bool encrypt_;
 };
 
-class InputBar : public QObject
+class InputBar final : public QObject
 {
     Q_OBJECT
     Q_PROPERTY(bool uploading READ uploading NOTIFY uploadingChanged)
@@ -208,7 +220,7 @@ private slots:
     void startTyping();
     void stopTyping();
 
-    void finalizeUpload(MediaUpload *upload, QString url);
+    void finalizeUpload(MediaUpload *upload, const QString &url);
     void removeRunUpload(MediaUpload *upload);
 
 signals:
@@ -220,6 +232,7 @@ signals:
 private:
     void emote(const QString &body, bool rainbowify);
     void notice(const QString &body, bool rainbowify);
+    void confetti(const QString &body, bool rainbowify);
     void command(const QString &name, QString args);
     void image(const QString &filename,
                const std::optional<mtx::crypto::EncryptedFile> &file,
@@ -256,6 +269,8 @@ private:
                const QSize &thumbnailDimensions,
                const QString &blurhash);
 
+    mtx::common::Relations generateRelations() const;
+
     void startUploadFromPath(const QString &path);
     void startUploadFromMimeData(const QMimeData &source, const QString &format);
     void startUpload(std::unique_ptr<QIODevice> dev, const QString &orgPath, const QString &format);
@@ -278,14 +293,6 @@ private:
     bool uploading_      = false;
     bool containsAtRoom_ = false;
 
-    struct DeleteLaterDeleter
-    {
-        void operator()(QObject *p)
-        {
-            if (p)
-                p->deleteLater();
-        }
-    };
     using UploadHandle = std::unique_ptr<MediaUpload, DeleteLaterDeleter>;
     std::vector<UploadHandle> unconfirmedUploads;
     std::vector<UploadHandle> runningUploads;
