@@ -1,5 +1,6 @@
 // SPDX-FileCopyrightText: 2021 Nheko Contributors
 // SPDX-FileCopyrightText: 2022 Nheko Contributors
+// SPDX-FileCopyrightText: 2023 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -11,7 +12,7 @@
 
 class TimelineModel;
 
-class Permissions : public QObject
+class Permissions final : public QObject
 {
     Q_OBJECT
 
@@ -25,11 +26,17 @@ public:
     Q_INVOKABLE bool canRedact();
     Q_INVOKABLE bool canChange(int eventType);
     Q_INVOKABLE bool canSend(int eventType);
+    Q_INVOKABLE int defaultLevel();
+    Q_INVOKABLE int redactLevel();
+    Q_INVOKABLE int changeLevel(int eventType);
+    Q_INVOKABLE int sendLevel(int eventType);
 
     Q_INVOKABLE bool canPingRoom();
 
     void invalidate();
 
+    const mtx::events::state::PowerLevels &powerlevelEvent() const { return pl; };
+
 private:
     QString roomId_;
     mtx::events::state::PowerLevels pl;
