@@ -1,5 +1,6 @@
 // SPDX-FileCopyrightText: 2021 Nheko Contributors
 // SPDX-FileCopyrightText: 2022 Nheko Contributors
+// SPDX-FileCopyrightText: 2023 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -12,14 +13,15 @@
 #include <mtx/events.hpp>
 #include <mtx/events/presence.hpp>
 
-class PresenceEmitter : public QObject
+class PresenceEmitter final : public QObject
 {
     Q_OBJECT
 
 public:
     PresenceEmitter(QObject *p = nullptr)
       : QObject(p)
-    {}
+    {
+    }
 
     void sync(const std::vector<mtx::events::Event<mtx::events::presence::Presence>> &presences);
 
