@@ -233,9 +233,13 @@ TimelineViewManager::showEvent(const QString &room_id, const QString &event_id)
     if (auto room = rooms_->getRoomById(room_id)) {
         auto exWin = MainWindow::instance()->windowForRoom(room_id);
         if (exWin) {
+            exWin->setVisible(true);
+            exWin->raise();
             exWin->requestActivate();
-        } else if (rooms_->currentRoom() != room) {
+        } else {
             rooms_->setCurrentRoom(room_id);
+            MainWindow::instance()->setVisible(true);
+            MainWindow::instance()->raise();
             MainWindow::instance()->requestActivate();
             nhlog::ui()->info("Activated room {}", room_id.toStdString());
         }
@@ -525,11 +529,12 @@ TimelineViewManager::forwardMessageToRoom(mtx::events::collections::TimelineEven
 
     std::visit(
       [room](auto e) {
-          if constexpr (mtx::events::message_content_to_type<decltype(e.content)> ==
-                        mtx::events::EventType::RoomMessage) {
+          constexpr auto type = mtx::events::message_content_to_type<decltype(e.content)>;
+          if constexpr (type == mtx::events::EventType::RoomMessage ||
+                        type == mtx::events::EventType::Sticker) {
               e.content.relations.relations.clear();
               removeReplyFallback(e);
-              room->sendMessageEvent(e.content, mtx::events::EventType::RoomMessage);
+              room->sendMessageEvent(e.content, type);
           }
       },
       *e);
