@@ -1,5 +1,6 @@
 // SPDX-FileCopyrightText: 2021 Nheko Contributors
 // SPDX-FileCopyrightText: 2022 Nheko Contributors
+// SPDX-FileCopyrightText: 2023 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -20,10 +21,12 @@
 #include <QTemporaryFile>
 #endif
 
+#include "ChatPage.h"
 #include "EventAccessors.h"
 #include "Logging.h"
 #include "MatrixClient.h"
 #include "timeline/TimelineModel.h"
+#include "timeline/TimelineViewManager.h"
 
 MxcMediaProxy::MxcMediaProxy(QObject *parent)
   : QMediaPlayer(parent)
@@ -42,10 +45,15 @@ MxcMediaProxy::MxcMediaProxy(QObject *parent)
     });
     connect(this,
             qOverload<const QString &, const QVariant &>(&MxcMediaProxy::metaDataChanged),
-            [this](QString t, QVariant) {
+            [this](const QString &t, const QVariant &) {
                 if (t == QMediaMetaData::Orientation)
                     emit orientationChanged();
             });
+
+    connect(ChatPage::instance()->timelineManager()->rooms(),
+            &RoomlistModel::currentRoomChanged,
+            this,
+            &MxcMediaProxy::pause);
 }
 void
 MxcMediaProxy::setVideoSurface(QAbstractVideoSurface *surface)
@@ -122,7 +130,7 @@ MxcMediaProxy::startDownload()
             QByteArray ba = device.readAll();
             std::string temp(ba.constData(), ba.size());
             temp = mtx::crypto::to_string(mtx::crypto::decrypt_file(temp, encryptionInfo.value()));
-            buffer.setData(temp.data(), temp.size());
+            buffer.setData(temp.data(), static_cast<int>(temp.size()));
         } else {
             buffer.setData(device.readAll());
         }
