@@ -1,5 +1,6 @@
 // SPDX-FileCopyrightText: 2021 Nheko Contributors
 // SPDX-FileCopyrightText: 2022 Nheko Contributors
+// SPDX-FileCopyrightText: 2023 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -9,7 +10,7 @@
 
 #include <mtxclient/http/client.hpp>
 
-class UIA : public QObject
+class UIA final : public QObject
 {
     Q_OBJECT
 
@@ -20,17 +21,18 @@ public:
 
     UIA(QObject *parent = nullptr)
       : QObject(parent)
-    {}
+    {
+    }
 
     mtx::http::UIAHandler genericHandler(QString context);
 
     QString title() const { return title_; }
 
 public slots:
-    void continuePassword(QString password);
-    void continueEmail(QString email);
-    void continuePhoneNumber(QString countryCode, QString phoneNumber);
-    void submit3pidToken(QString token);
+    void continuePassword(const QString &password);
+    void continueEmail(const QString &email);
+    void continuePhoneNumber(const QString &countryCode, const QString &phoneNumber);
+    void submit3pidToken(const QString &token);
     void continue3pidReceived();
 
 signals:
