@@ -1,5 +1,6 @@
 // SPDX-FileCopyrightText: 2021 Nheko Contributors
 // SPDX-FileCopyrightText: 2022 Nheko Contributors
+// SPDX-FileCopyrightText: 2023 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -21,24 +22,27 @@
 #include "timeline/TimelineViewManager.h"
 #include "ui/UIA.h"
 
-UserProfile::UserProfile(QString roomid,
-                         QString userid,
+UserProfile::UserProfile(const QString &roomid,
+                         const QString &userid,
                          TimelineViewManager *manager_,
                          TimelineModel *parent)
   : QObject(parent)
   , roomid_(roomid)
   , userid_(userid)
+  , globalAvatarUrl{QLatin1String("")}
   , manager(manager_)
   , model(parent)
 {
-    globalAvatarUrl = QLatin1String("");
-
     connect(this,
             &UserProfile::globalUsernameRetrieved,
             this,
             &UserProfile::setGlobalUsername,
             Qt::QueuedConnection);
-    connect(this, &UserProfile::verificationStatiChanged, &UserProfile::updateVerificationStatus);
+    connect(this,
+            &UserProfile::verificationStatiChanged,
+            this,
+            &UserProfile::updateVerificationStatus,
+            Qt::QueuedConnection);
 
     if (isGlobalUserProfile()) {
         getGlobalProfileData();
@@ -186,9 +190,7 @@ UserProfile::fetchDeviceList(const QString &userID)
       [other_user_id = userID.toStdString(), this](const UserKeyCache &,
                                                    mtx::http::RequestErr err) {
           if (err) {
-              nhlog::net()->warn("failed to query device keys: {},{}",
-                                 mtx::errors::to_string(err->matrix_error.errcode),
-                                 static_cast<int>(err->status_code));
+              nhlog::net()->warn("failed to query device keys: {}", *err);
           }
 
           // Ensure local key cache is up to date
@@ -199,9 +201,7 @@ UserProfile::fetchDeviceList(const QString &userID)
                 std::string local_user_id = utils::localUser().toStdString();
 
                 if (err) {
-                    nhlog::net()->warn("failed to query device keys: {},{}",
-                                       mtx::errors::to_string(err->matrix_error.errcode),
-                                       static_cast<int>(err->status_code));
+                    nhlog::net()->warn("failed to query device keys: {}", *err);
                 }
 
                 emit verificationStatiChanged();
@@ -257,9 +257,7 @@ UserProfile::updateVerificationStatus()
           [this, deviceInfo](const mtx::responses::QueryDevices &allDevs,
                              mtx::http::RequestErr err) mutable {
               if (err) {
-                  nhlog::net()->warn("failed to query devices: {} {}",
-                                     err->matrix_error.error,
-                                     static_cast<int>(err->status_code));
+                  nhlog::net()->warn("failed to query device keys: {}", *err);
                   this->deviceList_.queueReset(std::move(deviceInfo));
                   emit devicesChanged();
                   return;
@@ -325,13 +323,13 @@ UserProfile::startChat()
 }
 
 void
-UserProfile::changeUsername(QString username)
+UserProfile::changeUsername(const QString &username)
 {
     if (isGlobalUserProfile()) {
         // change global
         http::client()->set_displayname(username.toStdString(), [](mtx::http::RequestErr err) {
             if (err) {
-                nhlog::net()->warn("could not change username");
+                nhlog::net()->warn("could not change username: {}", *err);
                 return;
             }
         });
@@ -349,12 +347,12 @@ UserProfile::changeUsername(QString username)
 }
 
 void
-UserProfile::changeDeviceName(QString deviceID, QString deviceName)
+UserProfile::changeDeviceName(const QString &deviceID, const QString &deviceName)
 {
     http::client()->set_device_name(
       deviceID.toStdString(), deviceName.toStdString(), [this](mtx::http::RequestErr err) {
           if (err) {
-              nhlog::net()->warn("could not change device name");
+              nhlog::net()->warn("could not change device name: {}", *err);
               return;
           }
           refreshDevices();
@@ -372,7 +370,7 @@ UserProfile::verify(QString device)
 }
 
 void
-UserProfile::unverify(QString device)
+UserProfile::unverify(const QString &device)
 {
     cache::markDeviceUnverified(userid_.toStdString(), device.toStdString());
 }
@@ -430,14 +428,14 @@ UserProfile::changeAvatar()
        room_id  = roomid_.toStdString(),
        content = std::move(bin)](const mtx::responses::ContentURI &res, mtx::http::RequestErr err) {
           if (err) {
-              nhlog::ui()->error("Failed to upload image", err->matrix_error.error);
+              nhlog::ui()->error("Failed to upload image: {}", *err);
               return;
           }
 
           if (isGlobalUserProfile()) {
               http::client()->set_avatar_url(res.content_uri, [this](mtx::http::RequestErr err) {
                   if (err) {
-                      nhlog::ui()->error("Failed to set user avatar url", err->matrix_error.error);
+                      nhlog::ui()->error("Failed to set user avatar url: {}", *err);
                   }
 
                   isLoading_ = false;
@@ -459,14 +457,14 @@ UserProfile::changeAvatar()
 void
 UserProfile::updateRoomMemberState(mtx::events::state::Member member)
 {
-    http::client()->send_state_event(
-      roomid_.toStdString(),
-      http::client()->user_id().to_string(),
-      member,
-      [](mtx::responses::EventId, mtx::http::RequestErr err) {
-          if (err)
-              nhlog::net()->error("Failed to update room member state : ", err->matrix_error.error);
-      });
+    http::client()->send_state_event(roomid_.toStdString(),
+                                     http::client()->user_id().to_string(),
+                                     member,
+                                     [](mtx::responses::EventId, mtx::http::RequestErr err) {
+                                         if (err)
+                                             nhlog::net()->error(
+                                               "Failed to update room member state: {}", *err);
+                                     });
 }
 
 void
