@@ -1,5 +1,6 @@
 // SPDX-FileCopyrightText: 2021 Nheko Contributors
 // SPDX-FileCopyrightText: 2022 Nheko Contributors
+// SPDX-FileCopyrightText: 2023 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -32,14 +33,15 @@ Q_ENUM_NS(Status)
 class DeviceVerificationFlow;
 class TimelineViewManager;
 
-class UserProfileFetchProxy : public QObject
+class UserProfileFetchProxy final : public QObject
 {
     Q_OBJECT
 
 public:
     UserProfileFetchProxy(QObject *p = nullptr)
       : QObject(p)
-    {}
+    {
+    }
 
 signals:
     void profileFetched(mtx::responses::Profile);
@@ -58,7 +60,8 @@ public:
       , verification_status(verification_status_)
       , lastIp(lastIp_)
       , lastTs(lastTs_)
-    {}
+    {
+    }
     DeviceInfo(const QString deviceID,
                const QString displayName,
                verification::Status verification_status_)
@@ -66,10 +69,12 @@ public:
       , display_name(displayName)
       , verification_status(verification_status_)
       , lastTs(0)
-    {}
+    {
+    }
     DeviceInfo()
       : verification_status(verification::UNVERIFIED)
-    {}
+    {
+    }
 
     QString device_id;
     QString display_name;
@@ -79,7 +84,7 @@ public:
     qlonglong lastTs;
 };
 
-class DeviceInfoModel : public QAbstractListModel
+class DeviceInfoModel final : public QAbstractListModel
 {
     Q_OBJECT
 public:
@@ -116,7 +121,7 @@ private:
     friend class UserProfile;
 };
 
-class UserProfile : public QObject
+class UserProfile final : public QObject
 {
     Q_OBJECT
     Q_PROPERTY(QString displayName READ displayName NOTIFY displayNameChanged)
@@ -130,8 +135,8 @@ class UserProfile : public QObject
     Q_PROPERTY(bool isSelf READ isSelf CONSTANT)
     Q_PROPERTY(TimelineModel *room READ room CONSTANT)
 public:
-    UserProfile(QString roomid,
-                QString userid,
+    UserProfile(const QString &roomid,
+                const QString &userid,
                 TimelineViewManager *manager_,
                 TimelineModel *parent = nullptr);
 
@@ -148,7 +153,7 @@ public:
     TimelineModel *room() const { return model; }
 
     Q_INVOKABLE void verify(QString device = QLatin1String(""));
-    Q_INVOKABLE void unverify(QString device = QLatin1String(""));
+    Q_INVOKABLE void unverify(const QString &device = QLatin1String(""));
     Q_INVOKABLE void fetchDeviceList(const QString &userID);
     Q_INVOKABLE void refreshDevices();
     Q_INVOKABLE void banUser();
@@ -157,8 +162,8 @@ public:
     Q_INVOKABLE void kickUser();
     Q_INVOKABLE void startChat();
     Q_INVOKABLE void startChat(bool encryptionEnabled);
-    Q_INVOKABLE void changeUsername(QString username);
-    Q_INVOKABLE void changeDeviceName(QString deviceID, QString deviceName);
+    Q_INVOKABLE void changeUsername(const QString &username);
+    Q_INVOKABLE void changeDeviceName(const QString &deviceID, const QString &deviceName);
     Q_INVOKABLE void changeAvatar();
     Q_INVOKABLE void openGlobalProfile();
 
