@@ -1,5 +1,6 @@
 // SPDX-FileCopyrightText: 2021 Nheko Contributors
 // SPDX-FileCopyrightText: 2022 Nheko Contributors
+// SPDX-FileCopyrightText: 2023 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -141,7 +142,8 @@ parseSDP(const std::string &sdp, GstWebRTCSDPType type)
 {
     GstSDPMessage *msg;
     gst_sdp_message_new(&msg);
-    if (gst_sdp_message_parse_buffer((guint8 *)sdp.c_str(), sdp.size(), msg) == GST_SDP_OK) {
+    if (gst_sdp_message_parse_buffer((guint8 *)sdp.c_str(), static_cast<guint>(sdp.size()), msg) ==
+        GST_SDP_OK) {
         return gst_webrtc_session_description_new(type, msg);
     } else {
         nhlog::ui()->error("WebRTC: failed to parse remote session description");
@@ -299,10 +301,9 @@ testPacketLoss(gpointer G_GNUC_UNUSED)
 void
 setWaitForKeyFrame(GstBin *decodebin G_GNUC_UNUSED, GstElement *element, gpointer G_GNUC_UNUSED)
 {
-    if (!std::strcmp(
-          gst_plugin_feature_get_name(GST_PLUGIN_FEATURE(gst_element_get_factory(element))),
-          "rtpvp8depay"))
-        g_object_set(element, "wait-for-keyframe", TRUE, nullptr);
+    // Unconditionally enable keyframe wait and requesting keyframes, so that we do that for
+    // every decode, not just vp8 decoding
+    g_object_set(element, "wait-for-keyframe", TRUE, "request-keyframe", TRUE, nullptr);
 }
 
 GstElement *
@@ -371,9 +372,12 @@ getResolution(GstElement *pipe, const gchar *elementName, const gchar *padName)
 std::pair<int, int>
 getPiPDimensions(const std::pair<int, int> &resolution, int fullWidth, double scaleFactor)
 {
-    int pipWidth  = fullWidth * scaleFactor;
-    int pipHeight = static_cast<double>(resolution.second) / resolution.first * pipWidth;
-    return {pipWidth, pipHeight};
+    double pipWidth  = fullWidth * scaleFactor;
+    double pipHeight = static_cast<double>(resolution.second) / resolution.first * pipWidth;
+    return {
+      static_cast<int>(std::ceil(pipWidth)),
+      static_cast<int>(std::ceil(pipHeight)),
+    };
 }
 
 void
@@ -576,53 +580,81 @@ getMediaAttributes(const GstSDPMessage *sdp,
 }
 
 bool
-WebRTCSession::havePlugins(bool isVideo, std::string *errorMessage)
+WebRTCSession::havePlugins(bool isVideo, bool isX11Screenshare, std::string *errorMessage)
 {
     if (!initialised_ && !init(errorMessage))
         return false;
-    if (!isVideo && haveVoicePlugins_)
-        return true;
-    if (isVideo && haveVideoPlugins_)
+    if (haveVoicePlugins_ && (!isVideo || haveVideoPlugins_) &&
+        (!isX11Screenshare || haveX11ScreensharePlugins_))
         return true;
 
-    const gchar *voicePlugins[] = {"audioconvert",
-                                   "audioresample",
-                                   "autodetect",
-                                   "dtls",
-                                   "nice",
-                                   "opus",
-                                   "playback",
-                                   "rtpmanager",
-                                   "srtp",
-                                   "volume",
-                                   "webrtc",
-                                   nullptr};
+    static constexpr std::initializer_list<const char *> audio_elements = {
+      "audioconvert",
+      "audioresample",
+      "autoaudiosink",
+      "capsfilter",
+      "decodebin",
+      "opusenc",
+      "queue",
+      "rtpopuspay",
+      "volume",
+      "webrtcbin",
+    };
 
-    const gchar *videoPlugins[] = {
-      "compositor", "opengl", "qmlgl", "rtp", "videoconvert", "vpx", nullptr};
+    static constexpr std::initializer_list<const char *> video_elements = {
+      "compositor",
+      "glcolorconvert",
+      "glsinkbin",
+      "glupload",
+      "qmlglsink",
+      "rtpvp8pay",
+      "tee",
+      "videoconvert",
+      "videoscale",
+      "vp8enc",
+    };
+    static constexpr std::initializer_list<const char *> screenshare_elements = {
+      "ximagesink",
+      "ximagesrc",
+    };
 
-    std::string strError("Missing GStreamer plugins: ");
-    const gchar **needed  = isVideo ? videoPlugins : voicePlugins;
-    bool &havePlugins     = isVideo ? haveVideoPlugins_ : haveVoicePlugins_;
-    havePlugins           = true;
+    std::string strError("Missing GStreamer elements: ");
     GstRegistry *registry = gst_registry_get();
-    for (guint i = 0; i < g_strv_length((gchar **)needed); i++) {
-        GstPlugin *plugin = gst_registry_find_plugin(registry, needed[i]);
-        if (!plugin) {
-            havePlugins = false;
-            strError += std::string(needed[i]) + " ";
-            continue;
+
+    auto check_plugins = [&strError,
+                          registry](const std::initializer_list<const char *> &elements) {
+        bool havePlugins = true;
+        for (const auto &element : elements) {
+            GstPluginFeature *plugin =
+              gst_registry_find_feature(registry, element, GST_TYPE_ELEMENT_FACTORY);
+            if (!plugin) {
+                havePlugins = false;
+                strError += std::string(element) + " ";
+                continue;
+            }
+            gst_object_unref(plugin);
         }
-        gst_object_unref(plugin);
-    }
-    if (!havePlugins) {
+
+        return havePlugins;
+    };
+
+    haveVoicePlugins_ = check_plugins(audio_elements);
+
+    // check both elements at once
+    if (isVideo)
+        haveVideoPlugins_ = check_plugins(video_elements);
+    if (isX11Screenshare)
+        haveX11ScreensharePlugins_ = check_plugins(screenshare_elements);
+
+    if (!haveVoicePlugins_ || (isVideo && !haveVideoPlugins_) ||
+        (isX11Screenshare && !haveX11ScreensharePlugins_)) {
         nhlog::ui()->error(strError);
         if (errorMessage)
             *errorMessage = strError;
         return false;
     }
 
-    if (isVideo) {
+    if (isVideo || isX11Screenshare) {
         // load qmlglsink to register GStreamer's GstGLVideoItem QML type
         GstElement *qmlglsink = gst_element_factory_make("qmlglsink", nullptr);
         gst_object_unref(qmlglsink);
@@ -699,6 +731,15 @@ WebRTCSession::acceptOffer(const std::string &sdp)
     return true;
 }
 
+bool
+WebRTCSession::acceptNegotiation(const std::string &sdp)
+{
+    nhlog::ui()->debug("WebRTC: received negotiation offer:\n{}", sdp);
+    if (state_ == State::DISCONNECTED)
+        return false;
+    return false;
+}
+
 bool
 WebRTCSession::acceptAnswer(const std::string &sdp)
 {
@@ -1118,7 +1159,7 @@ WebRTCSession::end()
 #else
 
 bool
-WebRTCSession::havePlugins(bool, std::string *)
+WebRTCSession::havePlugins(bool, bool, std::string *)
 {
     return false;
 }
@@ -1138,6 +1179,12 @@ WebRTCSession::createOffer(webrtc::CallType, uint32_t)
 }
 // clang-format on
 
+bool
+WebRTCSession::acceptNegotiation(const std::string &)
+{
+    return false;
+}
+
 bool
 WebRTCSession::acceptOffer(const std::string &)
 {
@@ -1153,7 +1200,8 @@ WebRTCSession::acceptAnswer(const std::string &)
 void
 WebRTCSession::acceptICECandidates(
   const std::vector<mtx::events::voip::CallCandidates::Candidate> &)
-{}
+{
+}
 
 bool
 WebRTCSession::isMicMuted() const
@@ -1169,10 +1217,12 @@ WebRTCSession::toggleMicMute()
 
 void
 WebRTCSession::toggleLocalPiP()
-{}
+{
+}
 
 void
 WebRTCSession::end()
-{}
+{
+}
 
 #endif
